<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use Magento\User\Model\UserFactory;
use Magento\User\Model\ResourceModel\User\CollectionFactory;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory as ProductCollectionFactory;
use Magento\Catalog\Helper\Output as OutputHelper;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Helper\Data as HelperData;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;

class History extends Action
{
    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var UserFactory
     */
    protected $userFactory;

    /**
     * @var CollectionFactory
     */
    protected $userCollectionFactory;

    /**
     * @var TimezoneInterface
     */
    protected $timezone;

    /**
     * @var ProductCollectionFactory
     */
    protected $productCollectionFactory;

    /**
     * @var OutputHelper
     */
    protected $outputHelper;

    /**
     * @var ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var HelperData
     */
    protected $helperData;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * @var array
     */
    protected $users;

    /**
     * @var array
     */
    protected $products = [];

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param UserFactory $userFactory
     * @param CollectionFactory $userCollectionFactory
     * @param TimezoneInterface $timezone
     * @param ProductCollectionFactory $productCollectionFactory
     * @param OutputHelper $outputHelper
     * @param ProductRepositoryInterface $productRepository
     * @param HelperData $helperData
     * @param ApiSession $apiSession
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        UserFactory $userFactory,
        CollectionFactory $userCollectionFactory,
        TimezoneInterface $timezone,
        ProductCollectionFactory $productCollectionFactory,
        OutputHelper $outputHelper,
        ProductRepositoryInterface $productRepository,
        HelperData $helperData,
        ApiSession $apiSession,
        StoreHelper $storeHelper
    ) {
        $this->apiManager = $apiManager;
        $this->userFactory = $userFactory;
        $this->userCollectionFactory = $userCollectionFactory;
        $this->timezone = $timezone;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->outputHelper = $outputHelper;
        $this->productRepository = $productRepository;
        $this->helperData = $helperData;
        $this->apiSession = $apiSession;
        $this->storeHelper = $storeHelper;
        parent::__construct($context);
    }

    /**
     * Execute action
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        $result = [
            'success' => true,
            'entries' => [],
            'next_page_token' => '',
        ];

        try {
            $user = $this->apiSession->getCurrentUser();
            $regionIndependentLanguage = $this->apiSession->getRegionIndependentLanguage();
            
            $history = $this->apiManager->getHistory($this->getParams(), $regionIndependentLanguage);

            if (isset($history['histories']) &&
                is_array($history['histories']) &&
                !empty($history['histories'])
            ) {
                $this->collectProducts($history['histories']);
                $entries = $this->groupHistory($history['histories']);
                $result = [
                    'success' => true,
                    'entries' => $entries,
                    'next_page_token' => $history['continuationToken'] ?? '',
                ];
            }
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }

    /**
     * Group history by timestamp, user, and action
     *
     * @param array $history
     * @return array
     */
    protected function groupHistory($history)
    {
        /**
         * group by timestamp, user, and action
         */
        $entries = [];
        $storeId = $this->getRequest()->getParam('store_id', 0);
        $mappingSettings = $this->helperData->getMappingSettings();
        $mapping = [
            'page title' => $mappingSettings['page_title'],
            'page description' => $mappingSettings['page_description'],
            'product description' => $mappingSettings['product_description'],
            'excerpt' => $mappingSettings['short_product_description']
        ];
        
        if (isset($mappingSettings['open_graph'])) {
            $mapping['open graph text'] = $mappingSettings['open_graph'];
        }

        foreach ($history as $entry) {
            if (!isset($mapping[$entry['textType']])) {
                continue;
            }

            $product = $this->products[$entry['recordId']]['product'] ?? null;
            if (!$product) {
                try {
                    $product = $this->productRepository->getById(
                        $entry['recordId'],
                        false,
                        $storeId
                    );
                } catch (NoSuchEntityException $e) {
                    continue;
                }
            }
            
            $value = $this->outputHelper->productAttribute(
                $product,
                $entry['value'],
                $mapping[$entry['textType']]
            );
            
            $date = $this->timezone->date($entry['timestamp']);
            $timestamp = $date->format('Y-m-d H:i');

            if (!isset($entries[
                $timestamp .
                $entry['recordId'] .
                $entry['editor'] .
                $entry['action']
            ])) {
                $userName = $this->getUserNameByEmail($entry['editor']);

                $entries[
                    $timestamp .
                    $entry['recordId'] .
                    $entry['editor'] .
                    $entry['action']
                ] = [
                    'timestamp' => $date->format('M d, Y') .' | '. $date->format('H:i'),
                    'actionDisplay' => $userName ? str_replace(
                        $entry['editor'],
                        $userName,
                        $entry['actionDisplay']
                    ) : $entry['actionDisplay'],
                    'productName' => $this->products[$entry['recordId']]['name'] ?? '',
                    'productId' => $entry['recordId'],
                    'storeId' => $storeId,
                    'textEntries' => []
                ];
            }

            if (!isset($entries[
                $timestamp .
                $entry['recordId'] .
                $entry['editor'] .
                $entry['action']
            ]['textEntries'][$entry['textType']])) {
                $entries[
                    $timestamp .
                    $entry['recordId'] .
                    $entry['editor'] .
                    $entry['action']
                ]['textEntries'][$entry['textType']] = [];
            }
            
            $entries[
                $timestamp .
                $entry['recordId'] .
                $entry['editor'] .
                $entry['action']
            ]['textEntries'][$entry['textType']][] = $value;
        }

        return $entries;
    }

    /**
     * Get params for API request
     *
     * @return mixed[]
     */
    private function getParams()
    {
        $params = $this->getRequest()->getParams();
        $data = [
            'type' => 'Product',
            'storeId' => $params['store_id'],
            'recordId' => $params['product_id'],
        ];

        $language = $this->storeHelper->getFormattedLanguage($params['store_id']);
        $data['language'] = $language;

        if (isset($params['start_date']) && $params['start_date']) {
            $startDate = $this->timezone->convertConfigTimeToUtc($params['start_date'] . ' 00:00:00');
            $data['startDate'] = $startDate;
        }

        if (isset($params['end_date']) && $params['end_date']) {
            $endDate = $this->timezone->convertConfigTimeToUtc($params['end_date'] . ' 23:59:59');
            $data['endDate'] = $endDate;
        }

        if (isset($params['user']) && $params['user']) {
            $model = $this->userFactory->create();
            $user = $model->load($params['user']);

            if ($user->getId()) {
                $data['userName'] = $user->getEmail();
            }
        }

        if (isset($params['token']) && $params['token']) {
            $data['continuationToken'] = $params['token'];
        }
        
        return $data;
    }

    /**
     * Get user name by email
     *
     * @param string $email
     * @return string
     */
    protected function getUserNameByEmail($email)
    {
        try {
            $users = $this->getUsers();

            foreach ($users as $user) {
                if ($user->getEmail() == $email) {
                    return $user->getFirstname() . ' ' . $user->getLastname();
                }
            }
        } catch (\Exception $e) {
            return '';
        }
    }

    /**
     * Get users
     *
     * @return \Magento\User\Model\ResourceModel\User\Collection
     */
    protected function getUsers()
    {
        if (!$this->users) {
            $collection = $this->userCollectionFactory->create();
            $this->users = $collection;
        }

        return $this->users;
    }

    /**
     * Collect products
     *
     * @param array $history
     * @return void
     */
    protected function collectProducts($history)
    {
        $storeId = $this->getRequest()->getParam('store_id', 0);
        
        try {
            $productIds = array_unique(array_column($history, 'recordId'));
        } catch (\Exception $e) {
            return;
        }

        $collection = $this->productCollectionFactory->create();
        $collection->setStoreId($storeId);
        $collection->addAttributeToSelect('name');
        $collection->addFieldToFilter('entity_id', ['in' => $productIds]);

        foreach ($collection as $product) {
            if (!isset($this->products[$product->getId()])) {
                $productName = $this->outputHelper->productAttribute(
                    $product,
                    $product->getName(),
                    'name'
                );
                $this->products[$product->getId()] = [
                    'name' => $productName,
                    'product' => $product
                ];
            }
        }
    }
}
