<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\Api\Keywords as ApiKeywords;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Helper\Keywords as KeywordsHelper;
use WriteTextAI\WriteTextAI\Model\AiProductManager;

class GetData extends Action
{
    /**
     * @var ApiKeywords
     */
    protected $apiKeywords;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var KeywordsHelper
     */
    protected $keywordsHelper;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiKeywords $apiKeywords
     * @param Premium $premium
     * @param SettingManager $settingManager
     * @param ApiSession $apiSession
     * @param KeywordsHelper $keywordsHelper
     * @param AiProductManager $aiProductManager
     */
    public function __construct(
        Context $context,
        ApiKeywords $apiKeywords,
        Premium $premium,
        SettingManager $settingManager,
        ApiSession $apiSession,
        KeywordsHelper $keywordsHelper,
        AiProductManager $aiProductManager
    ) {
        $this->apiKeywords = $apiKeywords;
        $this->premium = $premium;
        $this->settingManager = $settingManager;
        $this->apiSession = $apiSession;
        $this->keywordsHelper = $keywordsHelper;
        $this->aiProductManager = $aiProductManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        try {
            $keywords = $this->getRequest()->getParam('keywords', []);
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');
            $location = $this->getRequest()->getParam('location');
            $filterByCompetition = $this->getRequest()->getParam('competition', []);
            $filterBySearchVolumeMinimum = $this->getRequest()->getParam('search_volume_minimum');
            $filterBySearchVolumeMaximum = $this->getRequest()->getParam('search_volume_maximum');
            $competitionSort = $this->getRequest()->getParam('competition_sort');
            $volumeSort = $this->getRequest()->getParam('volume_sort');
            $keywordSort = $this->getRequest()->getParam('keyword_sort');
            $url = $this->getRequest()->getParam('url');

            $userLocale = $this->apiSession->getCurrentUser()->getInterfaceLocale();
            $regionIndependentLanguage = $this->apiSession->getRegionIndependentLanguage();

            $rules = $this->settingManager->getSettings('rules');

            $concattedKeywords = [];
            if ($keywords) {
                $concattedKeywords = array_map(function ($text) use ($rules) {
                    return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
                }, $keywords);
            }

            $ideas = [];
            $rankedData = [];
            $showSelectedRefresh = false;
            $showCompetitorRefresh = false;
            $showIdeasRefresh = false;
            $showRankedRefresh = false;
            $newKeywordExists = false;
            $newTargetExists = false;
            $targetRemoved = false;
            try {
                $ideas = $this->apiKeywords->getKeywordsData(
                    $storeId,
                    $productId,
                    $location,
                    $regionIndependentLanguage,
                    $concattedKeywords,
                    $url,
                    $filterByCompetition,
                    $filterBySearchVolumeMinimum,
                    $filterBySearchVolumeMaximum,
                    $competitionSort,
                    $volumeSort,
                    $keywordSort
                );

                $newKeywordExists = $this->checkNewKeywordExists($ideas);
                $newTargetExists = $this->checkNewTargetExists($ideas, $keywords);
                $targetRemoved = $this->checkTargetRemoved($ideas, $keywords);

                if (isset($ideas['competitor_keywords']['keywords']) &&
                    is_array($ideas['competitor_keywords']['keywords'])
                ) {
                    $showCompetitorRefresh = true;
                    $showSelectedRefresh = true;
                }

                if (isset($ideas['result']) &&
                    is_array($ideas['result'])
                ) {
                    $showIdeasRefresh = true;
                }

                if (isset($ideas['ranked']) &&
                    is_array($ideas['ranked'])
                ) {
                    $showRankedRefresh = true;
                }
            } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
                if ($e->getCode() === 404) {
                    $rankedData = $this->apiKeywords->getRanked(
                        $storeId,
                        $productId,
                        $location,
                        $regionIndependentLanguage,
                        $url
                    );

                    if (isset($rankedData['keywords']) &&
                        is_array($rankedData['keywords'])
                    ) {
                        $showRankedRefresh = true;
                    }
                } else {
                    throw $e;
                }
            }

            $savedKeywords = [];
            if (isset($ideas['saved_keywords'])) {
                foreach ($ideas['saved_keywords'] as $savedKeyword) {
                    $savedKeywords[] = $this->keywordsHelper->getSavedKeywordData($savedKeyword, $ideas);
                }
            }
            
            $currDateTime = (new \DateTime())->format(\Magento\Framework\Stdlib\DateTime::DATETIME_PHP_FORMAT);

            $aiProduct = $this->aiProductManager->getProduct($productId, $storeId);
                
            $result = [
                'success' => true,
                'api_response' => $ideas,
                'stale' => $ideas['stale'] ?? false,
                'new_keyword_exists' => $newKeywordExists,
                'new_target_exists' => $newTargetExists,
                'target_removed' => $targetRemoved,
                'ranked_api_response' => $rankedData,
                'keywords' => $this->getSelectedKeywordsData($ideas, $keywords),
                'currently_ranking_keywords' => $ideas['ranked'] ?? $rankedData ?? [],
                'competitor_keywords' => $ideas['competitor_keywords'] ?? [],
                'saved_keywords' => $savedKeywords,
                'ideas' => $ideas['result'] ?? [],
                'serp_date' => isset($ideas['keywords_date']) ? sprintf(
                    __("SERP data as of %s"),
                    $this->keywordsHelper->formatDate($ideas['keywords_date'], $userLocale)
                ) : '',
                'total_page' => isset($ideas['result_count']) ? ceil($ideas['result_count'] / 10) : 0,
                'show_selected_refresh' => $showSelectedRefresh,
                'show_competitor_refresh' => $showCompetitorRefresh,
                'show_ideas_refresh' => $showIdeasRefresh,
                'show_ranked_refresh' => $showRankedRefresh,
                'ranked_date' => isset($ideas['ranked']['date']) ?
                    $this->keywordsHelper->formatDate($ideas['ranked']['date'], $userLocale) : '',
                'competitor_date' => isset($ideas['competitor_keywords']['date']) ?
                    $this->keywordsHelper->formatDate($ideas['competitor_keywords']['date'], $userLocale) : '',
                'ideas_date' => isset($ideas['keywords_date']) ?
                    $this->keywordsHelper->formatDate($ideas['keywords_date'], $userLocale) : '',
                'refresh' => $aiProduct->getId() ? ($aiProduct->getRefreshAt() ?
                    $aiProduct->getRefreshAt() <= $currDateTime : false) : false
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }

    /**
     * Check if new target exists
     *
     * @param array $ideas
     * @param array $keywords
     * 
     * @return bool
     */
    private function checkNewTargetExists($ideas, $keywords)
    {
        if (isset($ideas['keywords']) &&
            is_array($ideas['keywords']) &&
            is_array($keywords) &&
            !empty($keywords)
        ) {
            foreach ($keywords as $keyword) {
                $exists = false;
                foreach ($ideas['keywords'] as $idea) {
                    if (isset($idea['keyword']) && $idea['keyword'] === $keyword) {
                        $exists = true;
                        break;
                    }
                }
                if (!$exists) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check if target removed
     *
     * @param array $ideas
     * @param array $keywords
     * 
     * @return bool
     */
    private function checkTargetRemoved($ideas, $keywords)
    {
        if (isset($ideas['keywords']) &&
            is_array($ideas['keywords'])
        ) {
            foreach ($ideas['keywords'] as $idea) {
                if (!in_array($idea['keyword'], $keywords) &&
                    !in_array($idea['keyword'], $ideas['saved_keywords'])
                ) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check if new keyword exists
     *
     * @param array $ideas
     * @return bool
     */
    private function checkNewKeywordExists($ideas)
    {
        if (isset($ideas['keywords']) &&
            is_array($ideas['keywords']) &&
            isset($ideas['saved_keywords']) &&
            is_array($ideas['saved_keywords'])
        ) {
            foreach ($ideas['saved_keywords'] as $savedKeyword) {
                $exists = false;
                foreach ($ideas['keywords'] as $keyword) {
                    if (isset($keyword['keyword']) && $keyword['keyword'] === $savedKeyword) {
                        $exists = true;
                        break;
                    }
                }
                if (!$exists) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Get selected keywords data
     *
     * @param array $ideas
     * @param array $keywords
     * @return array
     */
    private function getSelectedKeywordsData($ideas, $keywords)
    {
        if (isset($ideas['competitor_keywords']) &&
            isset($ideas['competitor_keywords']['keywords'])
        ) {
            foreach ($keywords as &$keyword) {
                $keyword = [
                    'text' => $keyword
                ];
                foreach ($ideas['competitor_keywords']['keywords'] as $idea) {
                    if ($keyword['text'] === $idea['keyword']) {
                        $keyword['search_volume'] = $idea['search_volume'] ?? __('No data');
                        $keyword['competition'] = $idea['competition'] ?? __('No data');
                        $keyword['rank_group'] = __('No data');
                        $keyword['intent'] = __('No data');
                    }
                }
            }
        }

        $combinedIdeas = array_merge(
            $ideas['keywords'] ?? [],
            $ideas['ranked']['keywords'] ?? []
        );
        foreach ($keywords as &$keyword) {
            if (!is_array($keyword)) {
                $keyword = [
                    'text' => $keyword
                ];
            }
            foreach ($combinedIdeas as $idea) {
                if ($keyword['text'] === $idea['keyword']) {
                    $keyword['search_volume'] = $idea['search_volume'] ?? __('No data');
                    $keyword['competition'] = $idea['competition'] ?? __('No data');
                    $keyword['rank_group'] = $idea['rank_group'] ?? __('No data');
                    $keyword['intent'] = $idea['intent'] ?? __('No data');
                    $keyword['serp_infos'] = $idea['serp_infos'] ?? [];
                }
            }
        }

        return $keywords;
    }
}
