<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\Api\Keywords as ApiKeywords;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;
use WriteTextAI\WriteTextAI\Model\AiProductManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Helper\Keywords as KeywordsHelper;

class Ideas extends Action
{
    /**
     * @var ApiKeywords
     */
    protected $apiKeywords;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var KeywordsHelper
     */
    protected $keywordsHelper;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiKeywords $apiKeywords
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     * @param AiProductManager $aiProductManager
     * @param Premium $premium
     * @param SettingManager $settingManager
     * @param ApiSession $apiSession
     * @param KeywordsHelper $keywordsHelper
     */
    public function __construct(
        Context $context,
        ApiKeywords $apiKeywords,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager,
        AiProductManager $aiProductManager,
        Premium $premium,
        SettingManager $settingManager,
        ApiSession $apiSession,
        KeywordsHelper $keywordsHelper
    ) {
        $this->apiKeywords = $apiKeywords;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
        $this->aiProductManager = $aiProductManager;
        $this->premium = $premium;
        $this->settingManager = $settingManager;
        $this->apiSession = $apiSession;
        $this->keywordsHelper = $keywordsHelper;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $keywords = $this->getRequest()->getParam('keywords', []);
            $manualKeywords = $this->getRequest()->getParam('manual_keywords', []);
            $productId = $this->getRequest()->getParam('product_id');
            $productName = $this->getRequest()->getParam('product_name');
            $storeId = $this->getRequest()->getParam('store_id');
            $location = $this->getRequest()->getParam('location');
            $filterByCompetition = $this->getRequest()->getParam('competition', []);
            $filterBySearchVolumeMinimum = $this->getRequest()->getParam('search_volume_minimum');
            $filterBySearchVolumeMaximum = $this->getRequest()->getParam('search_volume_maximum');
            $competitionSort = $this->getRequest()->getParam('competition_sort');
            $volumeSort = $this->getRequest()->getParam('volume_sort');
            $keywordSort = $this->getRequest()->getParam('keyword_sort');
            $page = $this->getRequest()->getParam('page');
            $pageSize = $this->getRequest()->getParam('page_size');
            $refresh = filter_var($this->getRequest()->getParam('refresh'), FILTER_VALIDATE_BOOLEAN);
            $url = $this->getRequest()->getParam('url');

            $regionIndependentLanguage = $this->apiSession->getRegionIndependentLanguage();

            $rules = $this->settingManager->getSettings('rules');

            $concattedKeywords = [];
            if ($keywords) {
                $concattedKeywords = array_map(function ($text) use ($rules) {
                    return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
                }, $keywords);
            }

            $concattedManualKeywords = [];
            if ($manualKeywords) {
                $concattedManualKeywords = array_map(function ($text) use ($rules) {
                    return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
                }, $manualKeywords);
            }

            $ideas = $this->apiKeywords->getKeywordIdeas(
                $storeId,
                $productId,
                $productName,
                $concattedKeywords,
                $concattedManualKeywords,
                $location,
                $regionIndependentLanguage,
                $url,
                $filterByCompetition,
                $filterBySearchVolumeMinimum,
                $filterBySearchVolumeMaximum,
                $competitionSort,
                $volumeSort,
                $keywordSort,
                $page,
                $pageSize,
                $refresh
            );

            if (isset($ideas['saved_keywords'])) {
                $savedKeywords = [];
                foreach ($ideas['saved_keywords'] as $savedKeyword) {
                    $savedKeywords[] = $this->keywordsHelper->getSavedKeywordData($savedKeyword, $ideas);
                }
            }

            if ($refresh) {
                $this->keywordIdeaFiltersManager->deleteFilters($storeId, $productId);
                $this->aiProductManager->setRefreshDate($productId, $storeId);
            } else {
                $this->keywordIdeaFiltersManager->saveProductKeywordIdeasFilters(
                    $storeId,
                    $productId,
                    $filterByCompetition,
                    $filterBySearchVolumeMinimum,
                    $filterBySearchVolumeMaximum,
                    $competitionSort,
                    $volumeSort,
                    $keywordSort
                );
            }
                
            $result = [
                'success' => true,
                'request_id' => $ideas['request_id'] ?? null,
                'saved_keywords' => $savedKeywords ?? [],
                'ideas' => $ideas['result'] ?? [],
                'total_page' => isset($ideas['result_count']) ? ceil($ideas['result_count'] / $pageSize) : 0,
                'api_response' => $ideas
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
