<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use WriteTextAI\WriteTextAI\Model\Config\Source\DisallowedHtml;
use Magento\Framework\Filter\FilterManager;

class Html extends AbstractHelper
{
    /**
     * @var FilterManager
     */
    protected $filterManager;

    /**
     * Constructor.
     * @param FilterManager $filterManager
     */
    public function __construct(
        FilterManager $filterManager
    ) {
        $this->filterManager = $filterManager;
    }

    /**
     * Remove disallowed html
     *
     * @param string $html
     * @return string
     */
    public function removeDisallowedHtml($html)
    {
        $html = $this->removeScriptAndStyleContent($html);
        $html = $this->removeDisallowedAttributes($html, DisallowedHtml::ALLOWED_ATTRIBUTES);
        $html = $this->removeDisallowedStyles($html, DisallowedHtml::ALLOWED_STYLES);

        $params = [
            'allowableTags' => DisallowedHtml::ALLOWED_TAGS,
            'escape' => false
        ];
        
        try {
            $html = $this->filterManager->stripTags($html, $params);
        } catch (\Exception $e) {
            return $html;
        }

        return $html;
    }

    /**
     * Convert encoding
     *
     * @param string $html
     *
     * @return string
     */
    private function convertEncoding($html)
    {
        return mb_convert_encoding($html, 'ISO-8859-1', 'UTF-8');
    }

    /**
     * Remove script and style content
     *
     * @param string $html
     * @return string
     */
    private function removeScriptAndStyleContent($html)
    {
        try {
            $dom = new \DOMDocument();
            $dom->loadHTML(
                $this->convertEncoding("<div>{$html}</div>"),
                LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD | LIBXML_NOWARNING | LIBXML_NOERROR
            );

            $xpath = new \DOMXPath($dom);

            foreach ($xpath->query('//script') as $node) {
                $node->nodeValue = '';
            }

            foreach ($xpath->query('//style') as $node) {
                $node->nodeValue = '';
            }

            $result = "";
            foreach ($dom->documentElement->childNodes as $child) {
                $result .= $dom->saveHTML($child);
            }
            
            return $result;
        } catch (\Exception $e) {
            return $html;
        }
    }

    /**
     * Remove disallowed attributes
     *
     * @param string $html
     * @param array $allowedAttributes
     * @return string
     */
    private function removeDisallowedAttributes($html, $allowedAttributes)
    {
        try {
            $dom = new \DOMDocument();
            $dom->loadHTML(
                $this->convertEncoding("<div>{$html}</div>"),
                LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD | LIBXML_NOWARNING | LIBXML_NOERROR
            );

            $xpath = new \DOMXPath($dom);

            foreach ($xpath->query('//@*') as $node) {
                if (!isset($allowedAttributes[$node->parentNode->nodeName]) ||
                    !in_array($node->nodeName, $allowedAttributes[$node->parentNode->nodeName])
                ) {
                    $node->parentNode->removeAttribute($node->nodeName);
                }
            }

            $result = "";
            foreach ($dom->documentElement->childNodes as $child) {
                $result .= $dom->saveHTML($child);
            }
            
            return $result;
        } catch (\Exception $e) {
            return $html;
        }
    }

    /**
     * Remove disallowed styles
     *
     * @param string $html
     * @param array $allowedStyles
     * @return string
     */
    private function removeDisallowedStyles($html, $allowedStyles)
    {
        try {
            $dom = new \DOMDocument();
            $dom->loadHTML(
                $this->convertEncoding("<div>{$html}</div>"),
                LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD | LIBXML_NOWARNING | LIBXML_NOERROR
            );

            $xpath = new \DOMXPath($dom);

            foreach ($xpath->query('//@style') as $node) {
                $styles = explode(';', (string) $node->nodeValue);
                $newStyles = [];

                foreach ($styles as $style) {
                    $style = trim($style);

                    if ($style) {
                        $style = explode(':', (string) $style);
                        $style[0] = trim($style[0]);
                        $style[1] = trim($style[1]);

                        if (isset($allowedStyles[$node->parentNode->nodeName]) && (
                                in_array($style[0], $allowedStyles[$node->parentNode->nodeName]) ||
                                in_array('*', $allowedStyles[$node->parentNode->nodeName])
                            )
                        ) {
                            $newStyles[] = $style[0] . ': ' . $style[1];
                        }
                    }
                }

                $node->nodeValue = implode('; ', (array) $newStyles);
            }

            $result = "";
            foreach ($dom->documentElement->childNodes as $child) {
                $result .= $dom->saveHTML($child);
            }
            
            return $result;
        } catch (\Exception $e) {
            return $html;
        }
    }
}
