<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model\Api;

use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;
use WriteTextAI\WriteTextAI\Model\Api\CurlBuilder;
use WriteTextAI\WriteTextAI\Helper\Store as StoreHelper;

/**
 * @api
 */
class Keywords
{
    /**
     * @var RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var CurlBuilder
     */
    protected $curlBuilder;

    /**
     * @var StoreHelper
     */
    protected $storeHelper;

    /**
     * @var string
     */
    protected $ipAddress = '';

    /**
     * Constructor
     *
     * @param RemoteAddress $remoteAddress
     * @param CurlBuilder $curlBuilder
     * @param StoreHelper $storeHelper
     */
    public function __construct(
        RemoteAddress $remoteAddress,
        CurlBuilder $curlBuilder,
        StoreHelper $storeHelper
    ) {
        $this->remoteAddress = $remoteAddress;
        $this->curlBuilder = $curlBuilder;
        $this->storeHelper = $storeHelper;
    }

    /**
     * Get keywords data on page load
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $location
     * @param string $acceptLanguage
     * @param array $keywords
     * @param string $url
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param string $competitionSort
     * @param string $volumeSort
     * @param string $keywordSort
     *
     * @return array
     */
    public function getKeywordsData(
        $storeId,
        $recordId,
        $location,
        $acceptLanguage,
        $keywords,
        $url,
        $filterByCompetition = [
            "LOW",
            "MEDIUM",
            "HIGH"
        ],
        $filterBySearchVolumeMinimum = 0,
        $filterBySearchVolumeMaximum = null,
        $competitionSort = '',
        $volumeSort = '',
        $keywordSort = ''
    ) {
        $sorting = [];
        if ($competitionSort != '') {
            $sorting[] = [
                "field" => "competition_index",
                "ascending" => $competitionSort == 'true' ? true : false
            ];
        }

        if ($volumeSort != '') {
            $sorting[] = [
                "field" => "search_volume",
                "ascending" => $volumeSort == 'true' ? true : false
            ];
        }

        if ($keywordSort != '') {
            $sorting[] = [
                "field" => "keyword",
                "ascending" => $keywordSort == 'true' ? true : false
            ];
        }

        $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "ipAddress" => $this->getIpAddress(),
            "location_code" => $location,
            "language_code" => $storeLanguage,
            "url" => $url,
            "refresh" => false,
            "nogenerate" => true,
            "performCompetitiveAnalysis" => false,
            "page" => 1,
            "pageSize" => 10,
            "filterByCompetition" => $filterByCompetition,
            "filterBySearchVolumeMinimum" => $filterBySearchVolumeMinimum,
            "sorting" => $sorting

        ];

        if (!empty($keywords)) {
            $body['targetKeywords'] = $keywords;
        }

        if ($filterBySearchVolumeMaximum) {
            $body['filterBySearchVolumeMaximum'] = $filterBySearchVolumeMaximum;
        }

        $response = $this->curlBuilder->getApiResponse(
            'text/KeywordAnalysis/Ideas',
            json_encode($body),
            'post',
            $acceptLanguage
        );

        return $response;
    }

    /**
     * Get currently ranking keywords
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $locationCode
     * @param string $acceptLanguage
     * @param string $url
     *
     * @return array
     */
    public function getRanked(
        $storeId,
        $recordId,
        $locationCode,
        $acceptLanguage,
        $url
    ) {
        $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "ipAddress" => $this->getIpAddress(),
            "url" => $url,
            "location_code" => $locationCode,
            "language_code" => $storeLanguage
        ];

        try {
            $response = $this->curlBuilder->getApiResponse(
                'text/KeywordAnalysis/Ranked',
                json_encode($body),
                'post',
                $acceptLanguage
            );
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Refresh target keywords
     *
     * @param string $storeId
     * @param string $productId
     * @param string $productName
     * @param array $keywords
     * @param array $targetKeywords
     * @param string $location
     * @param string $url
     * @param string $userLang
     *
     * @return array
     */
    public function refreshTargetKeywords(
        $storeId,
        $productId,
        $productName,
        $keywords,
        $targetKeywords,
        $location,
        $url,
        $userLang
    ) {
        $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

        $body = [
            "storeId" => $storeId,
            "recordId" => $productId,
            "ipAddress" => $this->getIpAddress(),
            "location_code" => $location,
            "language_code" => $storeLanguage,
            "url" => $url,
            "refresh" => false,
            "nogenerate" => true,
            "performCompetitiveAnalysis" => true,
            "productName" => $productName
        ];

        if (!empty($keywords)) {
            $body['text'] = $keywords;
        }

        if (!empty($targetKeywords)) {
            $body['targetKeywords'] = $targetKeywords;
        }

        $response = $this->curlBuilder->getApiResponse(
            'text/KeywordAnalysis/Ideas',
            json_encode($body),
            'post',
            $userLang
        );

        return $response;
    }

    /**
     * Refresh competitor keywords
     *
     * @param string $storeId
     * @param string $productId
     * @param string $productName
     * @param array $keywords
     * @param array $targetKeywords
     * @param string $location
     * @param string $url
     * @param string $userLang
     *
     * @return array
     */
    public function refreshCompetitorKeywords(
        $storeId,
        $productId,
        $productName,
        $keywords,
        $targetKeywords,
        $location,
        $url,
        $userLang
    ) {
        $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

        $body = [
            "storeId" => $storeId,
            "recordId" => $productId,
            "ipAddress" => $this->getIpAddress(),
            "location_code" => $location,
            "language_code" => $storeLanguage,
            "url" => $url,
            "refresh" => false,
            "nogenerate" => true,
            "performCompetitiveAnalysis" => true,
            "productName" => $productName
        ];

        if (!empty($keywords)) {
            $body['text'] = $keywords;
        }

        if (!empty($targetKeywords)) {
            $body['targetKeywords'] = $targetKeywords;
        }

        $response = $this->curlBuilder->getApiResponse(
            'text/KeywordAnalysis/Ideas',
            json_encode($body),
            'post',
            $userLang
        );

        return $response;
    }

    /**
     * AI-powered analysis
     *
     * @param string $storeId
     * @param string $productId
     * @param string $productName
     * @param array $keywords
     * @param array $targetKeywords
     * @param string $location
     * @param string $userLang
     * @param string $url
     *
     * @return array
     */
    public function aiPoweredAnalysis(
        $storeId,
        $productId,
        $productName,
        $keywords,
        $targetKeywords,
        $location,
        $userLang,
        $url
    ) {
        try {
            $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

            $body = [
                "storeId" => $storeId,
                "recordId" => $productId,
                "ipAddress" => $this->getIpAddress(),
                "location_code" => $location,
                "language_code" => $storeLanguage,
                "url" => $url,
                "refresh" => true,
                "nogenerate" => true,
                "performCompetitiveAnalysis" => true,
                "productName" => $productName
            ];

            if (!empty($keywords)) {
                $body['text'] = $keywords;
                $body['nogenerate'] = false;
            }
    
            if (!empty($targetKeywords)) {
                $body['targetKeywords'] = $targetKeywords;
            }

            $response = $this->curlBuilder->getApiResponse(
                'text/KeywordAnalysis/Ideas',
                json_encode($body),
                'post',
                $userLang
            );
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Get Keyword Ideas
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $productName
     * @param array $keywords
     * @param array $manualKeywords
     * @param string $location
     * @param string $acceptLanguage
     * @param string $url
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param bool $competitionSort
     * @param bool $volumeSort
     * @param bool $keywordSort
     * @param int $page
     * @param int $pageSize
     * @param bool $refresh
     *
     * @return array
     */
    public function getKeywordIdeas(
        $storeId,
        $recordId,
        $productName,
        $keywords,
        $manualKeywords,
        $location,
        $acceptLanguage,
        $url,
        $filterByCompetition = [
            "LOW",
            "MEDIUM",
            "HIGH"
        ],
        $filterBySearchVolumeMinimum = 0,
        $filterBySearchVolumeMaximum = null,
        $competitionSort = '',
        $volumeSort = '',
        $keywordSort = '',
        $page = 1,
        $pageSize = 10,
        $refresh = true
    ) {
        $sorting = [];
        if ($competitionSort != '') {
            $sorting[] = [
                "field" => "competition_index",
                "ascending" => $competitionSort == 'true' ? true : false
            ];
        }

        if ($volumeSort != '') {
            $sorting[] = [
                "field" => "search_volume",
                "ascending" => $volumeSort == 'true' ? true : false
            ];
        }

        if ($keywordSort != '') {
            $sorting[] = [
                "field" => "keyword",
                "ascending" => $keywordSort == 'true' ? true : false
            ];
        }

        $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);

        $body = [
            "storeId" => $storeId,
            "recordId" => $recordId,
            "ipAddress" => $this->getIpAddress(),
            "location_code" => $location,
            "language_code" => $storeLanguage,
            "url" => $url,
            "refresh" => $refresh,
            "nogenerate" => true,
            "performCompetitiveAnalysis" => false,
            "page" => $page,
            "pageSize" => $pageSize,
            "filterByCompetition" => $filterByCompetition,
            "filterBySearchVolumeMinimum" => $filterBySearchVolumeMinimum,
            "sorting" => $sorting
        ];

        if ($refresh) {
            $body['productName'] = $productName;

            if (!empty($manualKeywords)) {
                $body['text'] = $manualKeywords;
                $body['nogenerate'] = false;
            }
        }

        if (!empty($keywords)) {
            $body['targetKeywords'] = $keywords;
        }

        if ($filterBySearchVolumeMaximum) {
            $body['filterBySearchVolumeMaximum'] = $filterBySearchVolumeMaximum;
        }

        try {
            $response = $this->curlBuilder->getApiResponse(
                'text/KeywordAnalysis/Ideas',
                json_encode($body),
                'post',
                $acceptLanguage
            );
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Save keyword
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $keyword
     * @param string $location
     *
     * @return string
     */
    public function saveKeyword($storeId, $recordId, $keyword, $location)
    {
        try {
            $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);
            $response = $this->curlBuilder->getApiResponse('text/KeywordAnalysis/Ideas', json_encode([
                "storeId" => $storeId,
                "recordId" => $recordId,
                "ipAddress" => $this->getIpAddress(),
                "location_code" => $location,
                "language_code" => $storeLanguage,
                "saveAsKeyword" => $keyword
            ]), 'post');
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Remove keyword
     *
     * @param string $storeId
     * @param string $recordId
     * @param string $keyword
     * @param string $location
     *
     * @return string
     */
    public function removeKeyword($storeId, $recordId, $keyword, $location)
    {
        try {
            $storeLanguage =  $this->storeHelper->getFormattedRegionIndependentLanguage($storeId);
            $response = $this->curlBuilder->getApiResponse('text/KeywordAnalysis/Ideas', json_encode([
                "storeId" => $storeId,
                "recordId" => $recordId,
                "ipAddress" => $this->getIpAddress(),
                "location_code" => $location,
                "language_code" => $storeLanguage,
                "removeAsKeyword" => $keyword
            ]), 'post');
        } catch (\WriteTextAI\WriteTextAI\Exception\ApiException $e) {
            if ($e->getCode() === 404) {
                $response = [];
            } else {
                throw $e;
            }
        }

        return $response;
    }

    /**
     * Get IP address
     */
    public function getIpAddress()
    {
        if ($this->ipAddress === '') {
            $this->ipAddress = $this->remoteAddress->getRemoteAddress();
        }

        return $this->ipAddress;
    }
}