<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use WriteTextAI\WriteTextAI\Model\ProductKeywordCompetitorFiltersFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordCompetitorFiltersRepositoryInterface;

class KeywordCompetitorFiltersManager
{
    /**
     * @var ProductKeywordCompetitorFiltersFactory
     */
    protected $productKeywordCompetitorFiltersFactory;

    /**
     * @var ProductKeywordCompetitorFiltersRepositoryInterface
     */
    protected $productKeywordCompetitorFiltersRepository;

    /**
     * @param ProductKeywordCompetitorFiltersFactory $productKeywordCompetitorFiltersFactory
     * @param ProductKeywordCompetitorFiltersRepositoryInterface $productKeywordCompetitorFiltersRepository
     */
    public function __construct(
        ProductKeywordCompetitorFiltersFactory $productKeywordCompetitorFiltersFactory,
        ProductKeywordCompetitorFiltersRepositoryInterface $productKeywordCompetitorFiltersRepository
    ) {
        $this->productKeywordCompetitorFiltersFactory = $productKeywordCompetitorFiltersFactory;
        $this->productKeywordCompetitorFiltersRepository = $productKeywordCompetitorFiltersRepository;
    }

    /**
     * Get last used filters
     *
     * @param int $productId
     * @param int $storeId
     * @return array
     */
    public function getKeywordCompetitorFilters($productId, $storeId)
    {
        $filters = $this->productKeywordCompetitorFiltersFactory->create();

        $filters = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($filters->getId()) {
            return [
                'competition' => explode(',', (string) $filters->getCompetition()),
                'search_volume_minimum' => (int) $filters->getSearchVolumeMinimum(),
                'search_volume_maximum' => $filters->getSearchVolumeMaximum() === null
                    ? null : (int) $filters->getSearchVolumeMaximum(),
                'competition_sort' => $filters->getCompetitionSort()
                    ? (bool) $filters->getCompetitionSort() : '',
                'volume_sort' => $filters->getVolumeSort()
                    ? (bool) $filters->getVolumeSort() : '',
                'rank_sort' => $filters->getRankSort()
                    ? (bool) $filters->getRankSort() : '',
                'intent_sort' => $filters->getIntentSort()
                    ? (bool) $filters->getIntentSort() : ''
            ];
        } else {
            return [
                'competition' => [
                    "LOW",
                    "MEDIUM",
                    "HIGH"
                ],
                'search_volume_minimum' => 0,
                'search_volume_maximum' => null,
                'competition_sort' => '',
                'volume_sort' => '',
                'rank_sort' => '',
                'intent_sort' => ''
            ];
        }
    }

    /**
     * Save last used filters
     *
     * @param int $storeId
     * @param int $productId
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param string $competitionSort
     * @param string $volumeSort
     * @param string $rankSort
     * @param string $intentSort
     *
     * @return void
     */
    public function saveKeywordCompetitorFilters(
        $storeId,
        $productId,
        $filterByCompetition,
        $filterBySearchVolumeMinimum,
        $filterBySearchVolumeMaximum,
        $competitionSort,
        $volumeSort,
        $rankSort,
        $intentSort
    ) {
        $defaultcompetition = [
            "LOW",
            "MEDIUM",
            "HIGH"
        ];
        
        $filters = $this->productKeywordCompetitorFiltersFactory->create();

        $filterByCompetition = implode(',', (array) $filterByCompetition);

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setCompetition($filterByCompetition);
            $existing->setSearchVolumeMinimum($filterBySearchVolumeMinimum);
            $existing->setCompetitionSort($competitionSort);
            $existing->setVolumeSort($volumeSort);
            $existing->setRankSort($rankSort);
            $existing->setIntentSort($intentSort);
            $existing->setSearchVolumeMaximum(
                $filterBySearchVolumeMaximum === '' ? null : $filterBySearchVolumeMaximum
            );

            $filters = $existing;
        } else {
            // Record not found, create a new one
            $data = [
                "product_id" => $productId,
                "store_id" => $storeId,
                "competition" => $filterByCompetition,
                "search_volume_minimum" => $filterBySearchVolumeMinimum,
                "competition_sort" => $competitionSort,
                "volume_sort" => $volumeSort,
                "rank_sort" => $rankSort,
                "intent_sort" => $intentSort
            ];
            
            if ($filterBySearchVolumeMaximum) {
                $data['search_volume_maximum'] = $filterBySearchVolumeMaximum;
            }

            $filters->setData($data);
        }

        $this->productKeywordCompetitorFiltersRepository->save($filters);
    }

    /**
     * Delete filter
     *
     * @param int $storeId
     * @param int $productId
     *
     * @return void
     */
    public function deleteFilters($storeId, $productId)
    {
        $filters = $this->productKeywordCompetitorFiltersFactory->create();

        $filters = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($filters->getId()) {
            $this->productKeywordCompetitorFiltersRepository->delete($filters);
        }
    }
}
