<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use WriteTextAI\WriteTextAI\Model\ProductKeywordIdeasFiltersFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordIdeasFiltersRepositoryInterface;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class KeywordIdeaFiltersManager
{
    /**
     * @var ProductKeywordIdeasFiltersFactory
     */
    protected $productKeywordIdeasFiltersFactory;

    /**
     * @var ProductKeywordIdeasFiltersRepositoryInterface
     */
    protected $productKeywordIdeasFiltersRepository;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * @param ProductKeywordIdeasFiltersFactory $productKeywordIdeasFiltersFactory
     * @param ProductKeywordIdeasFiltersRepositoryInterface $productKeywordIdeasFiltersRepository
     * @param ScopeConfigInterface $scopeConfig
     * @param Settings $settings
     */
    public function __construct(
        ProductKeywordIdeasFiltersFactory $productKeywordIdeasFiltersFactory,
        ProductKeywordIdeasFiltersRepositoryInterface $productKeywordIdeasFiltersRepository,
        ScopeConfigInterface $scopeConfig,
        Settings $settings
    ) {
        $this->productKeywordIdeasFiltersFactory = $productKeywordIdeasFiltersFactory;
        $this->productKeywordIdeasFiltersRepository = $productKeywordIdeasFiltersRepository;
        $this->scopeConfig = $scopeConfig;
        $this->settings = $settings;
    }

    /**
     * Get available countries
     *
     * @return array
     */
    public function getAvailableCountries()
    {
        $locations = $this->settings->getSettings('locations');
        if (!is_array($locations)) {
            $locations = json_decode($locations, true);
        }

        return $locations;
    }

    /**
     * Get last used filters
     *
     * @param int $productId
     * @param int $storeId
     * @return array
     */
    public function getKeywordIdeasFilters($productId, $storeId)
    {
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $filters = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($filters->getId()) {
            return [
                'competition' => explode(',', (string) $filters->getCompetition()),
                'search_volume_minimum' => (int) $filters->getSearchVolumeMinimum(),
                'search_volume_maximum' => $filters->getSearchVolumeMaximum() === null
                    ? null : (int) $filters->getSearchVolumeMaximum(),
                'competition_sort' => $filters->getCompetitionSort()
                    ? (bool) $filters->getCompetitionSort() : '',
                'volume_sort' => $filters->getVolumeSort()
                    ? (bool) $filters->getVolumeSort() : '',
                'keyword_sort' => $filters->getKeywordSort()
                    ? (bool) $filters->getKeywordSort() : ''
            ];
        } else {
            return [
                'competition' => [
                    "LOW",
                    "MEDIUM",
                    "HIGH"
                ],
                'search_volume_minimum' => 0,
                'search_volume_maximum' => null,
                'competition_sort' => '',
                'volume_sort' => '',
                'keyword_sort' => true
            ];
        }
    }

    /**
     * Get default location
     *
     * @param int $storeId
     * @return string
     */
    public function getLocationCode($storeId)
    {
        $countryCode = $this->getCountryCode($storeId);
        $locations = $this->getAvailableCountries();

        foreach ($locations as $location) {
            if ($location['country_iso_code'] == $countryCode) {
                return $location['location_code'];
            }
        }

        return 2840; // Default to United States
    }

    /**
     * Get country code
     *
     * @param int $storeId
     * @return string
     */
    public function getCountryCode($storeId)
    {
        $countryCode =  $this->scopeConfig->getValue(
            'general/country/default',
            ScopeInterface::SCOPE_STORE,
            $storeId
        );

        return $countryCode;
    }

    /**
     * Save last used filters
     *
     * @param int $storeId
     * @param int $productId
     * @param array $filterByCompetition
     * @param int $filterBySearchVolumeMinimum
     * @param int $filterBySearchVolumeMaximum
     * @param string $competitionSort
     * @param string $volumeSort
     * @param string $keywordSort
     *
     * @return void
     */
    public function saveProductKeywordIdeasFilters(
        $storeId,
        $productId,
        $filterByCompetition,
        $filterBySearchVolumeMinimum,
        $filterBySearchVolumeMaximum,
        $competitionSort,
        $volumeSort,
        $keywordSort
    ) {
        $defaultcompetition = [
            "LOW",
            "MEDIUM",
            "HIGH"
        ];
        
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $filterByCompetition = implode(',', (array) $filterByCompetition);

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($existing->getId()) {
            // Update the existing record
            $existing->setCompetition($filterByCompetition);
            $existing->setSearchVolumeMinimum($filterBySearchVolumeMinimum);
            $existing->setCompetitionSort($competitionSort);
            $existing->setVolumeSort($volumeSort);
            $existing->setKeywordSort($keywordSort);
            $existing->setSearchVolumeMaximum(
                $filterBySearchVolumeMaximum === '' ? null : $filterBySearchVolumeMaximum
            );

            $filters = $existing;
        } else {
            // Record not found, create a new one
            $data = [
                "product_id" => $productId,
                "store_id" => $storeId,
                "competition" => $filterByCompetition,
                "search_volume_minimum" => $filterBySearchVolumeMinimum,
                "competition_sort" => $competitionSort,
                "volume_sort" => $volumeSort,
                "keyword_sort" => $keywordSort,
            ];
            
            if ($filterBySearchVolumeMaximum) {
                $data['search_volume_maximum'] = $filterBySearchVolumeMaximum;
            }

            $filters->setData($data);
        }

        $this->productKeywordIdeasFiltersRepository->save($filters);
    }

    /**
     * Delete filter
     *
     * @param int $storeId
     * @param int $productId
     *
     * @return void
     */
    public function deleteFilters($storeId, $productId)
    {
        $filters = $this->productKeywordIdeasFiltersFactory->create();

        $existing = $filters->getCollection()
            ->addFieldToFilter('product_id', $productId)
            ->addFieldToFilter('store_id', $storeId)
            ->getFirstItem();

        if ($existing->getId()) {
            $this->productKeywordIdeasFiltersRepository->delete($existing);
        }
    }
}
