<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\ViewModel;

use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Helper\Data;
use WriteTextAI\WriteTextAI\ViewModel\Settings;

class Tooltip implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    public const FAQ_URL = 'https://writetext.ai/frequently-asked-questions#credit-cost';

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Settings
     */
    protected $settings;

    /**
     * Constructor
     *
     * @param SettingManager $settingManager
     * @param Data $helper
     * @param Settings $settings
     */
    public function __construct(
        SettingManager $settingManager,
        Data $helper,
        Settings $settings
    ) {
        $this->settingManager = $settingManager;
        $this->helper = $helper;
        $this->settings = $settings;
    }

    /**
     * Get rules
     *
     * @return array
     */
    public function getRules()
    {
        return $this->settings->getSettings('rules');
    }

    /**
     * Get tooltip
     *
     * @param string $block
     * @return string
     */
    public function getTooltip($block)
    {
        $tooltip = '';
        switch ($block) {
            case 'reviewed':
                $tooltip = $this->getReviewedTooltip();
                break;
            case 'keywords':
                $tooltip = $this->getKeywordsTooltip();
                break;
            case 'attributes':
                $tooltip = $this->getAttributesTooltip();
                break;
            case 'tones-and-styles':
                $tooltip = $this->getTonesAndStylesTooltip();
                break;
            case 'audience':
                $tooltip = $this->getAudienceTooltip();
                break;
            case 'page-title':
                $tooltip = $this->getPageTitleTooltip();
                break;
            case 'product-description':
                $tooltip = $this->getProductDescriptionTooltip();
                break;
            case 'short-description':
                $tooltip = $this->getShortDescriptionTooltip();
                break;
            case 'open-graph':
                $tooltip = $this->getOpenGraphTooltip();
                break;
            case 'page-description':
                $tooltip = $this->getPageDescriptionTooltip();
                break;
            case 'reference-product':
                $tooltip = $this->getReferenceProductTooltip();
                break;
            case 'generate-rewrite':
                $tooltip = $this->getGenerateRewriteTooltip();
                break;
            case 'transfer':
                $tooltip = $this->getTransferTooltip();
                break;
            case 'preview-changes':
                $tooltip = $this->getPreviewChangesTooltip();
                break;
            case 'alt-text':
                $tooltip = $this->getAltTextTooltip();
                break;
            case 'my-ranking':
                $tooltip = $this->getMyRankingTooltip();
                break;
            case 'competitors-ranking':
                $tooltip = $this->getCompetitorsRankingTooltip();
                break;
            case 'manual-input':
                $tooltip = $this->getManualInputTooltip();
                break;
            case 'ideas':
                $tooltip = $this->getIdeasTooltip();
                break;
            case 'ai-powered':
                $tooltip = $this->getAiPoweredTooltip();
                break;
            case 'selected-keywords':
                $tooltip = $this->getSelectedKeywordsTooltip();
                break;
            case 'special-instructions':
                $tooltip = $this->getSpecialInstructionsTooltip();
                break;
        }

        return $tooltip;
    }

    /**
     * Get special instructions tooltip
     *
     * @return string
     */
    public function getSpecialInstructionsTooltip()
    {
        return __('Enter other details or instructions that you would like WriteText.ai'
            . ' to consider in generating text for the products you selected'
            . ' (e.g., add information on discounts, special sales, etc).'
            . ' This will not replace any details you may have added in the “Other product details”'
            . ' section in the individual product pages.'
            . ' Special instructions will be cleared at the end of a user session.');
    }

    /**
     * Get AI-powered tooltip
     *
     * @return string
     */
    public function getAiPoweredTooltip()
    {
        return __('Click "Start AI-powered Keyword Analysis" to automatically enhance your keyword strategy.'
            . ' WriteText.ai will retrieve available keyword data for applicable sections.'
            . ' Check the help text for each section or watch this explainer video to learn more.');
    }

    /**
     * Get manual input tooltip
     *
     * @return string
     */
    public function getManualInputTooltip()
    {
        return sprintf('<p>%s</p>', __('If you want to know the search volume'
                . ' and difficulty data for a certain keyword,'
                . ' manually type it here and click “Start AI-powered keyword analysis” at the top to retrieve data.'))
            . sprintf('<p>%s</p>', __('You can also manually add a keyword to be'
                . ' included in your text by typing it here and clicking the + sign.'));
    }

    /**
     * Get intent tooltip
     *
     * @return string
     */
    private function getIntentTooltip()
    {
        return sprintf('<strong>%s</strong>', __('Intent guide:'))
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Navigational'),
                __('Users have a specific website or web page in mind and are'
                    . ' using search engines to navigate there directly.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Informational'),
                __('Users seek information on a specific topic, intending to learn something.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Transactional'),
                __('Users aim to complete a transaction, such as making a purchase or signing up for a service.')
            )
            . sprintf(
                '<p><em>%s</em> - %s</p>',
                __('Commercial'),
                __('Users are researching products or services with the intention of making a purchase decision soon')
            );
    }

    /**
     * Get selected keywords tooltip
     *
     * @return string
     */
    public function getSelectedKeywordsTooltip()
    {
        return sprintf(
            '<p>%s</p>',
            __('If you have selected any, WriteText.ai will retrieve the SERP data for each keyword'
                . ' (which you can preview upon clicking the keyword)'
                . ' along with intent, search volume, and difficulty.')
        ) . $this->getIntentTooltip();
    }

    /**
     * Get my ranking tooltip
     *
     * @return string
     */
    public function getMyRankingTooltip()
    {
        return sprintf(
            '<p>%s</p>',
            __('If it’s the first time you’re doing an AI-powered keyword analysis on the site,'
                . ' WriteText.ai will retrieve ranking data for the whole domain'
                . ' (i.e., this page and your other product pages). If your page is ranking for any keyword/s,'
                . ' they will show here along with data on intent, search volume, and difficulty.')
        ) . $this->getIntentTooltip();
    }

    /**
     * Get competitors ranking tooltip
     *
     * @return string
     */
    public function getCompetitorsRankingTooltip()
    {
        return sprintf(
            '<p>%s</p>',
            __('Competitors are pages that are ranking for either the same'
                . ' keywords you’re currently ranking for or the keywords you selected'
                . ' to be included in your text. You can check which competitors are ranking'
                . ' for which keyword by clicking on the keyword. The keywords shown in this'
                . ' section are ones that your competing pages are ranking for but this product page is not.'
                . ' Data on search volume, intent, and difficulty are also displayed here.')
        ) . $this->getIntentTooltip();
    }

    /**
     * Get ideas tooltip
     *
     * @return string
     */
    public function getIdeasTooltip()
    {
        return __('Keyword ideas based on the keywords you selected to'
            . ' be included in the text or your own manually-added keywords.');
    }

    /**
     * Get reviewed tooltip
     *
     * @return string
     */
    public function getReviewedTooltip()
    {
        return sprintf('<p>%s</p>', __('Check this box to keep track of the products'
                . ' where you have reviewed the text. This is especially helpful if you'
                . ' have an internal workflow where text needs to go through a review'
                . ' process first before being published on the website. This checkbox'
                . ' does not affect the live content, it is only a classification.'))
            . sprintf('<p>%s</p>', __('You can filter reviewed products by'
                . ' selecting “Reviewed” under the “Filter by WriteText.ai status”'
                . ' dropdown in the product list.'));
    }

    /**
     * Get keywords tooltip
     *
     * @return string
     */
    public function getKeywordsTooltip()
    {
        $rules = $this->getRules();
        $maxKeywords = $rules['maxKeywords'] ?? '';

        return sprintf('<p class=\"heading\">%s</p>', __('Target keywords'))
            . sprintf('<p>%s</p>', sprintf(__('Click the "Do keyword analysis" button'
                . ' to add up to %s target keywords for your product page. This will also'
                . ' open the dashboard where you can get keyword ideas as well as data on'
                . ' search volume and difficulty from Google and other search sources.'
                . ' If there are no target keywords added, WriteText.ai will generate text'
                . ' based on the product name and selected product attributes.'), $maxKeywords))
            . sprintf('<p class=\"heading\">%s</p>', __('Semantic keywords'))
            . sprintf('<p>%s</p>', __('Semantic keywords are words or phrases that are'
                . ' closely related to a primary keyword or topic, which helps search engines'
                . ' and users understand the context and meaning of a piece of content.'
                . ' Using semantic keywords in your content can improve its relevance and'
                . ' quality. WriteText.ai automatically suggests semantic keywords related'
                . ' to your product name and target keywords. Click on the semantic keyword'
                . ' if you want it to be considered in generating your product texts.'))
            . sprintf('<p class=\"heading\">%s</p>', __('Keyword density'))
            . sprintf('<p>%s</p>', __('Keyword density refers to the percentage of times'
                . ' your target keyword or semantic keyword appears throughout the page,'
                . ' in comparison to the total number of words in it. In WriteText.ai,'
                . ' we only take into account the text visible on your page — which are'
                . ' the description or the short description. This means that the meta title,'
                . ' meta description, and Open Graph text are not counted. There may also be cases'
                . ' where you only use the product description in the page and not the product'
                . ' short description or vice versa. We advise you to leave the field empty'
                . ' for the text type you are not using so that you can get an accurate'
                . ' keyword density. You can see the keyword density percentage displayed'
                . ' next to every keyword in this section.'));
    }

    /**
     * Get attributes tooltip
     *
     * @return string
     */
    public function getAttributesTooltip()
    {
        return  sprintf("<p>%s</p>", __("Check the product attributes that you want to be"
                . " considered in generating text. You can also select your main product image here"
                . " and AI will analyze it in order to generate more accurate and relevant text."
                . " Please make sure that the image accurately represents the product (i.e., it is"
                . " not some kind of placeholder or a generic image for your shop)."))
            . sprintf("<p>%s</p>", __('There is no guarantee that a selected product attribute'
                . ' will always appear in the text, but it will help guide WriteText.ai'
                . ' in generating more relevant text.'))
            . sprintf("<p>%s</p>", __('If you want to add more information about the product,'
                . ' you can enter them in the “Other product details” section.'
                . ' For example, you can write:'))
            . sprintf("<p><em>%s</em>", __("“The shirt comes in a reusable shirt bag that can be used for traveling.”"))
            . sprintf("<br/><br/>%s", __(
                "WriteText.ai will then attempt to include the text or its meaning in the generated product text, e.g.:"
            ))
            . sprintf("<br/><br/><em>%s</em></p>", __(
                "“...Plus, its reusable bag makes it perfect for travel or to keep in storage when not in use!”"
            ));
    }

    /**
     * Get tones and styles tooltip
     *
     * @return string
     */
    public function getTonesAndStylesTooltip()
    {
        return __('<p>Select one or more tones to set the overall mood'
            . ' and attitude of the text, and then choose a style to'
            . ' set the voice and structure of the text.</p>');
    }

    /**
     * Get audience tooltip
     *
     * @return string
     */
    public function getAudienceTooltip()
    {
        return __('<p>Select the applicable audience for your product so WriteText.ai'
            . ' can generate text that will appeal to them.  If you don’t select an audience,'
            . ' the generated text will default to a “neutral” audience. WriteText.ai also'
            . ' suggests more specific target markets based on your keywords. Click on the'
            . ' audience to select it or type in your own custom audience in the box.</p>');
    }

    /**
     * Get meta title tooltip
     *
     * @return string
     */
    public function getPageTitleTooltip()
    {
        $rules = $this->getRules();
        $maxPageTitleLength = $rules['maxPageTitleLength'] ?? '';

        return sprintf(__('<p>WriteText.ai aims to generate a title with around'
            . ' %s characters, based on current SEO best practices. The current'
            . ' meta title saved for the product is displayed in the box on'
            . ' the right for your reference only.</p>'), $maxPageTitleLength);
    }

    /**
     * Get meta description tooltip
     *
     * @return string
     */
    public function getPageDescriptionTooltip()
    {
        $maxPageDescriptionLength = $this->helper->getMaxLengthSettings('page_description');

        return sprintf(__('<p>WriteText.ai aims to generate a description with'
            . ' around %s characters, based on current SEO best practices. The current'
            . ' meta description saved for the product is displayed in the box on'
            . ' the right for your reference only.</p>'), $maxPageDescriptionLength);
    }

    /**
     * Get description tooltip
     *
     * @return string
     */
    public function getProductDescriptionTooltip()
    {
        return __('<p>Please check your website to see if you are using the description or'
                . ' the short description field (or both) to ensure that you are generating the'
                . ' correct text. The current description saved for the product is displayed'
                . ' in the box on the right for your reference only.</p>')
            . __('<p>Target length<br/>')
            . __('Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.');
    }

    /**
     * Get short description tooltip
     *
     * @return string
     */
    public function getShortDescriptionTooltip()
    {
        return sprintf('<p>%s</p>', __('Please check your website to see if'
                . ' you are using the description or the short description field (or both)'
                . ' to ensure that you are generating the correct text. The current short'
                . ' description saved for the product is displayed in the box on'
                . ' the right for your reference only.'))
            . __('<p>Target length<br/>')
            . __('Indicate your target length by setting a minimum and maximum word count.'
                . ' WriteText.ai will aim to generate text within the number you set,'
                . ' but it may give you more or less words than expected from time to time.'
                . ' Note that your credit cost will depend on the target length you'
                . ' set so make sure to set a reasonable target range.');
    }

    /**
     * Get open graph tooltip
     *
     * @return string
     */
    public function getOpenGraphTooltip()
    {
        $maxOpenGraphLength = $this->helper->getMaxLengthSettings('open_graph');

        return sprintf(__('<p>This is the text that appears in the preview when you share'
            . ' the product page on social media sites like Facebook, Twitter, or LinkedIn.'
            . ' WriteText.ai aims to generate text with more or less %s characters,'
            . ' based on current best practices. The current Open Graph text saved for'
            . ' the product is displayed in the box on the right'
            . ' for your reference only.</p>'), $maxOpenGraphLength);
    }

    /**
     * Get transfer tooltip
     *
     * @return string
     */
    public function getTransferTooltip()
    {
        return sprintf("<p>%s</p>", __("When you're done generating and/or editing text,"
                . " you have the option to save the draft inside WriteText.ai or transfer"
                . " the text to Magento. Transferring your text to Magento will publish it"
                . " directly to the website. If the updates did not reflect in your page,"
                . " you may need to flush the cache or run reindexing. Please also make"
                . " sure that your product is enabled in the frontend."))
            . sprintf("<p>%s</p>", __("Note: Any media or directive you have inserted in"
                . " your current Magento text will be overwritten when you transfer from WriteText.ai."));
    }

    /**
     * Get reference product tooltip
     *
     * @return string
     */
    public function getReferenceProductTooltip()
    {
        return sprintf('<p>%s</p>', __("If you have already edited a certain product’s"
                . " texts to fit the specific style and structure that you want, you "
                . "can set it as a “Reference product” which WriteText.ai will use as the basis for generating text."
                . " When using a reference product, the individual settings for tone, style, and audience will be"
                . " ignored. Note that image alt text generation will not be affected by the reference product."))
            . sprintf('<p>%s</p>', __("Note: The texts that will be used as reference"
                . " are the ones currently in Magento. Only products with all text types saved"
                . " in Magento will be available for selection in this list."))
            . sprintf(__('The credit cost for generating text using a reference product will'
                . ' depend on the length of the reference text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on'
                . ' <a href="%s" target="_blank">how your credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get generate rewrite tooltip
     *
     * @return string
     */
    public function getGenerateRewriteTooltip()
    {
        return sprintf('<p class=\"heading\">%s</p>', __('Generate new text'))
            . sprintf('<p>%s</p>', __('Generate text for your selected text types based on the settings on this page.'))
            . sprintf('<p class=\"heading\">%s</p>', __('Rewrite existing'))
            . sprintf('<p>%s</p>', __('If any of your settings'
                . ' (e.g., keywords, product attributes, tone, style, and audiences)'
                . ' have changed but you don’t necessarily want to generate completely new text,'
                . ' you can use the Rewrite feature to keep the existing structure of the text but'
                . ' rewrite it to reflect any changes. Note that for the image alt text,'
                . ' only the product name and keywords are considered in the rewrite.'))
            . sprintf('<p>%s</p>', __('Note: Any media or directive you have inserted into the existing text'
                . ' will not be included in the rewritten version. The language of the text you'
                . ' are rewriting will be the same language used in the output.'))
            . sprintf(__('The credit cost for rewriting text will depend'
                . ' on the length of the original text and any formatting'
                . ' (e.g., bold, italics, numbered or bulleted lists) applied to it.'
                . ' For a more detailed explanation, you may read our FAQ section on <a href="%s" target="_blank">'
                . 'how your credit cost is calculated</a>.'), self::FAQ_URL);
    }

    /**
     * Get preview changes tooltip
     *
     * @return string
     */
    public function getPreviewChangesTooltip()
    {
        return __('Save your changes to preview how they will look on your website.');
    }

    /**
     * Get alt text tooltip
     *
     * @return string
     */
    public function getAltTextTooltip()
    {
        $altTextLength = $this->helper->getMaxLengthSettings('alt_text');

        return sprintf(__("Analyze your product image and automatically generate an alt image text"
            . " (incorporating your product name and keywords) to help improve your website's"
            . " accessibility and the way that search engines understand your content."
            . " WriteText.ai aims to generate an alt text with around %s characters based on current best practices."
            . " Note: We only allow up to 10 images here, including the product featured image."
            . " If you have more than 10 images uploaded to your product,"
            . " only the first 10 images in your listing will be included."), $altTextLength);
    }
}
