/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keyword-filters',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/competitors-ranking',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/my-ranking',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/ideas',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'mage/translate',
    'accordion'
], function (
    $,
    ko,
    Component,
    _,
    premium,
    prompt,
    totalCredits,
    keywords,
    product,
    keywordFilters,
    competitorsRankingComponent,
    myRankingComponent,
    ideasComponent,
    keywordProgress,
    keywordUtils,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            refreshMessage: $t("Keyword data are refreshed at the start of every month. The data you’re seeing might have changed since you last requested it. Click the “Start AI-powered keyword analysis” button to refresh data."),
            selectors: {
                tooltipSelector: '.writetextai-keyword-analysis-content .admin__field-tooltip-action',
                accordionSelector: '.writetextai-keyword-analysis-content .accordions'
            }
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            this.initTooltips();
            this.initAccordion();
            this.initDropdowns();

            $('.writetextai-keyword-analysis-content .header > *').applyBindings(this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([]);

            this.errorMessage = ko.computed(function () {
                return keywords.errorMessage();
            });

            this.successMessage = ko.computed(function () {
                return keywords.successMessage();
            });

            this.language = ko.computed(function () {
                return keywords.language();
            });

            this.country = ko.computed(function () {
                return keywords.country();
            });

            this.analysisInProgress = keywordUtils.analysisInProgress();

            return this;
        },

        /**
         * Initialize tooltips.
         */
        initTooltips: function () {
            $(self.selectors.tooltipSelector).dropdown({
                parent: '.admin__field-tooltip',
                autoclose: true
            });
        },

        /**
         * Initialize accordion.
         */
        initAccordion: function () {
            $(self.selectors.accordionSelector).each(function () {
                $(this).accordion({
                    active: [0, 1, 2, 3],
                    collapsible: true,
                    openedState: '_show',
                    closedState: '_hide',
                    multipleCollapsible: true
                });
            });
        },

        /**
         * Initialize dropdowns.
         */
        initDropdowns: function () {
            $('.filterable').dropdown({
                parent: '.filters',
                autoclose: true
            });
        },

        /**
         * Dismiss error message.
         *
         * @param {string} type
         * @returns {void}
         */
        dismissMessage: function (type = 'error') {
            switch (type) {
                case 'error':
                    keywords.errorMessage('');
                    break;
                case 'success':
                    keywords.successMessage('');
                    break;
            }
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword);
        },

        /**
         * Get initial data
         */
        getData: function () {
            keywords.errorMessage('');
            keywords.fetchingData(true);
            keywords.showSelectedRefresh(false);
            keywords.showCompetitorRefresh(false);
            keywords.showIdeasRefresh(false);
            keywords.showRankedRefresh(false);
            keywords.initialized(false);

            $.ajax({
                url: self.getDataUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keywords: keywords.targetKeywords()
                        .filter(keyword => self.keywordRemovable(keyword.text))
                        .map(keyword => keyword.text),
                    location: keywords.location(),
                    url: product.productUrl(),
                    competition: keywordFilters.ideasSortAndFilters().competition,
                    search_volume_minimum: keywordFilters.ideasSortAndFilters().search_volume_minimum,
                    search_volume_maximum: keywordFilters.ideasSortAndFilters().search_volume_maximum,
                    competition_sort: keywordFilters.ideasSortAndFilters().competition_sort,
                    volume_sort: keywordFilters.ideasSortAndFilters().volume_sort,
                    keyword_sort: keywordFilters.ideasSortAndFilters().keyword_sort
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    keywordProgress.selectedAndCompetitorLoading(false);
                    keywordProgress.ideasLoading(false);
                    keywords.fetchingData(false);
                    keywords.initialized(true);

                    /**
                     * Don't hide the loading mask if signalr is ongoing.
                     */
                    if (!(Object.keys(keywordProgress.status()).length > 0 &&
                        keywordProgress.allSectionsProgressVisible() &&
                        keywordProgress.status().status !== 'Completed'
                    )) {
                        keywordProgress.allSectionsLoading(false);
                        $('.writetextai-analysis-loading-mask').removeClass('show');
                    }

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (response.success) {
                        /* Update target keywords with new data */
                        let currentKeywords = [...keywords.targetKeywords()];
                        let updatedKeywords = currentKeywords.map(targetKeyword => {
                            let keyword = response.keywords.find(keyword => keyword.text === targetKeyword.text);

                            if (keyword) {
                                targetKeyword.competition = keyword.competition;
                                targetKeyword.search_volume = keyword.search_volume;
                                targetKeyword.rank_group = keyword.rank_group;
                                targetKeyword.intent = keyword.intent;
                                targetKeyword.serp_infos = keyword.serp_infos;
                            }

                            return targetKeyword;
                        });
                        keywords.targetKeywords([]);
                        keywords.targetKeywords(self.arrangeRanking(updatedKeywords));
                        /* End update target keywords */

                        keywords.myRanking(response.currently_ranking_keywords);
                        keywords.competitorRanking(self.filterOrganic(response.competitor_keywords));
                        keywords.manualInput(response.saved_keywords);
                        keywords.suggestedKeywords(response.ideas);
                        keywords.serpDate(response.serp_date);
                        keywords.totalPages(response.total_page);
                        keywords.showSelectedRefresh(response.show_selected_refresh);
                        keywords.showCompetitorRefresh(response.show_competitor_refresh);
                        keywords.showIdeasRefresh(response.show_ideas_refresh);
                        keywords.showRankedRefresh(response.show_ranked_refresh);
                        keywords.rankedDate(response.ranked_date);
                        keywords.competitorDate(response.competitor_date);
                        keywords.ideasDate(response.ideas_date);
                        keywords.stale(response.stale);
                        keywords.newKeywordExists(response.new_keyword_exists);
                        keywords.newTargetExists(response.new_target_exists);
                        keywords.targetRemoved(response.target_removed);
                        keywords.needRefresh(response.refresh);

                        /**
                         * Reapply filters and sorting.
                         * Will not work if called right after updating the keywords.
                         */
                        keywordFilters.rankedSortAndFilters.valueHasMutated();
                        keywordFilters.competitorSortAndFilters.valueHasMutated();

                        if (response.refresh) {
                            keywords.errorMessage(self.refreshMessage);
                        }
                    } else {
                        keywords.errorMessage(response.message);
                    }
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        keywords.errorMessage(error);
                    }
                }
            });
        },

        /**
         * AI Powered Analysis.
         */
        aiPoweredAnalysis: function () {
            keywords.successMessage('');
            keywordProgress.allSectionsLoading(true);
            keywordProgress.allSectionsProgressVisible(true);
            keywords.errorMessage('');
            competitorsRankingComponent.prototype.resetSortingAndFilters();
            myRankingComponent.prototype.resetSortingAndFilters();
            ideasComponent.prototype.resetSortingAndFilters();
            keywordFilters.competitorSortAndFilters({
                competition: ['LOW', 'MEDIUM', 'HIGH'],
                search_volume_minimum: 0,
                search_volume_maximum: null,
                volume_sort: '',
                competition_sort: '',
                intent_sort: '',
                rank_sort: ''
            });
            keywordFilters.rankedSortAndFilters({
                competition: ['LOW', 'MEDIUM', 'HIGH'],
                search_volume_minimum: 0,
                search_volume_maximum: null,
                volume_sort: '',
                competition_sort: '',
                intent_sort: '',
                rank_sort: ''
            });
            keywordFilters.ideasSortAndFilters({
                competition: ['LOW', 'MEDIUM', 'HIGH'],
                search_volume_minimum: 0,
                search_volume_maximum: null,
                volume_sort: '',
                competition_sort: '',
                intent_sort: '',
                rank_sort: ''
            });

            $.ajax({
                url: self.aiPoweredUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    product_name: product.productName(),
                    store_id: product.storeId(),
                    manual_keywords: keywords.manualInput().map(input => input.keyword),
                    keywords: keywords.targetKeywords()
                        .map(input => input.text)
                        .filter(keyword => self.keywordRemovable(keyword)),
                    location: keywords.location(),
                    product_url: product.productUrl(),
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showAnalysisLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (!response.success) {
                        keywords.errorMessage(response.message);
                        keywordProgress.selectedAndCompetitorLoading(false);
                        keywordProgress.allSectionsLoading(false);
                        keywordProgress.ideasLoading(false);
                        $('.writetextai-analysis-loading-mask').removeClass('show');
                    }
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        keywords.errorMessage(error);
                    }
                }
            });
        },

        /**
         * Filter organic serp data
         *
         * @param {object} data
         * @returns {object}
         */
        filterOrganic: function (data) {
            if (!data.keywords) {
                return data;
            }

            data.keywords.forEach(keyword => {
                if (!keyword.serp_infos) {
                    return;
                }

                keyword.serp_infos = keyword.serp_infos.filter(obj => obj.type === 'organic');
            });

            return data;
        },

        /**
         * Arrange SERP ranking
         *
         * @param {object} keywords
         * @returns {object}
         */
        arrangeRanking: function (keywords) {
            if (!keywords) {
                return keywords;
            }

            keywords.forEach(keyword => {
                if (!keyword.serp_infos) {
                    return;
                }
                
                let serpInfos = [...keyword.serp_infos];
                let topFive = [...serpInfos];
                let productUrl = window.location.hostname;

                if (!topFive.some(obj => obj.domain === productUrl)) {
                    let firstUrlAppearance = serpInfos.findIndex(obj => obj.domain === productUrl);

                    if (firstUrlAppearance !== -1) {
                        topFive.pop();
                        topFive.push(serpInfos[firstUrlAppearance]);
                    }
                }

                keyword.serp_infos = topFive;
            });

            return keywords;
        }
    });
});
