/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/keyword-filters',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/helpers',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    product,
    premium,
    prompt,
    totalCredits,
    keywordFilters,
    audience,
    keywordProgress,
    helperUtils,
    keywordUtils,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generalMessage: $t('A system error has occurred. Please click the "Start AI-powered analysis" button again. If the issue persists, please contact our support team at support@writetext.ai.'),
            volumeSort: '',
            competitionSort: '',
            intentSort: '',
            rankSort: '',
            competitorKeywords: [],
            canShowMore: false,
            filterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            defaultFilterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            filterBySearchVolume: '{"min": 0, "max": null}',
            defaultFilterBySearchVolume: '{"min": 0, "max": null}'
        },

        gettingData: null,
        isFilterChanged: true,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.keyword-analysis-competitors-ranking > *').applyBindings(this);

            keywordUtils.initToggleSerpPopup('.keyword-analysis-competitors-ranking', '.competitors-popup');

            keywords.competitorRanking.subscribe(function (value) {
                if (value.keywords) {
                    this.competitorKeywords(value.keywords);
                } else {
                    this.competitorKeywords([]);
                }
            }, this);

            keywordFilters.competitorSortAndFilters.subscribe(function (value) {
                this.isFilterChanged = false;

                this.filterByCompetition(value.competition);
                this.filterBySearchVolume(`{"min": ${value.search_volume_minimum}, "max": ${value.search_volume_maximum}}`);

                switch (true) {
                    case value.volume_sort !== '':
                        this.sortBySearchVolume(value.volume_sort);
                        break;
                    case value.competition_sort !== '':
                        this.sortByDifficulty(value.competition_sort);
                        break;
                    case value.intent_sort !== '':
                        this.sortByIntent(value.intent_sort);
                        break;
                    case value.rank_sort !== '':
                        this.sortByRank(value.rank_sort);
                        break;
                    default:
                        this.sortByKeyword();
                        break;
                }

                this.isFilterChanged = true;
            }, this);

            this.competitorKeywords.subscribe(function (competitorKeywords) {
                if (competitorKeywords.length > 0) {
                    this.canShowMore(competitorKeywords.length > 5);
                    $(".keyword-analysis-competitors-ranking .keyword-row").hide();
                    $(".keyword-analysis-competitors-ranking .keyword-row").slice(0, 5).show();
                } else {
                    this.canShowMore(false);
                }
            }, this);

            this.filterBySearchVolume.subscribe(function (filterBySearchVolume) {
                var volumesObject = JSON.parse(filterBySearchVolume),
                    filterByCompetition = this.filterByCompetition() || [];

                this.saveFilter();

                if (!keywords.competitorRanking().keywords) {
                    return;
                }
                
                this.competitorKeywords(keywords.competitorRanking().keywords.filter(function (keyword) {
                    return filterByCompetition.includes(keyword.competition) && (
                        keyword.search_volume >= volumesObject.min && (
                            volumesObject.max === null ||
                            keyword.search_volume <= volumesObject.max
                        )
                    );
                }, this));
            }, this);

            this.filterByCompetition.subscribe(function (filterByCompetition) {
                filterByCompetition = filterByCompetition || [];
                
                this.saveFilter();

                if (!keywords.competitorRanking().keywords) {
                    return;
                }

                this.competitorKeywords(keywords.competitorRanking().keywords.filter(function (keyword) {
                    var volumesObject = JSON.parse(this.filterBySearchVolume());

                    return filterByCompetition.includes(keyword.competition) && (
                        keyword.search_volume >= volumesObject.min && (
                            volumesObject.max === null ||
                            keyword.search_volume <= volumesObject.max
                        )
                    );
                }, this));
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'volumeSort',
                    'competitionSort',
                    'intentSort',
                    'rankSort',
                    'competitorKeywords',
                    'canShowMore',
                    'filterByCompetition',
                    'filterBySearchVolume'
                ]);

            this.hasFilter = ko.computed(function () {
                var filterByCompetition = this.filterByCompetition() || [];

                return filterByCompetition.length !== this.defaultFilterByCompetition.length ||
                    this.filterBySearchVolume() !== this.defaultFilterBySearchVolume;
            }, this);

            this.hasSort = ko.computed(function () {
                return this.volumeSort() !== '' ||
                    this.competitionSort() !== '' ||
                    this.intentSort() !== '' ||
                    this.rankSort() !== '';
            }, this);

            this.targetKeywords = ko.computed(function () {
                return keywords.targetKeywords();
            });

            this.date = ko.computed(function () {
                return keywords.competitorRanking().date || '';
            });

            this.isKeywordsMax = ko.computed(function () {
                var keywords = this.targetKeywords().filter(function (keyword) {
                        return keywordUtils.keywordRemovable(keyword.text);
                }),
                    keywordsCount = keywords ? keywords.length : 0;

                return keywordsCount >= this.rules.maxKeywords;
            }, this);
            
            this.status = ko.computed(function () {
                return keywordProgress.status();
            });

            this.showProgress = ko.computed(function () {
                return Object.keys(this.status()).length > 0 &&
                    keywordProgress.selectedAndCompetitorProgressVisible() &&
                    this.status().status !== 'Completed';
            }, this);

            this.getMessage = keywordUtils.getMessage(this);

            this.getCount = keywordUtils.getCount(this);

            this.getPercent = keywordUtils.getPercent(this);

            this.isLoading = ko.computed(function () {
                return keywordProgress.selectedAndCompetitorLoading();
            });

            this.analysisInProgress = keywordUtils.analysisInProgress();

            this.keywordsChangeOngoing = ko.computed(function () {
                return keywords.keywordsChangeOngoing();
            });

            this.isSectionDisabled = ko.computed(function () {
                return keywords.ideasSortFilterOngoing() ||
                    keywords.ideasShowMoreOngoing() ||
                    keywords.keywordsChangeOngoing() ||
                    keywords.manualKeywordsChangeOngoing() ||
                    keywordProgress.ideasLoading();
            });

            this.showRefresh = ko.computed(function () {
                return keywords.showCompetitorRefresh();
            }, this);

            this.noResultText = ko.computed(function () {
                return '<p>' + helperUtils.sprintf(
                            $t('No competitor keywords found as of %s'),
                            keywords.competitorDate()
                        ) + '</p>'
                    + '<br>'
                    + '<p>' + $t('Select or manually type other keywords and try again.') + '</p>';
            }); 

            return this;
        },

        /**
         * Get domain
         * 
         * @param {String} domain
         * @returns {String}
         */
        getDomain: function (domain) {
            if (!domain.startsWith('http://') && !domain.startsWith('https://')) {
                domain = 'https://' + domain;
            }
            return domain;
        },

        /**
         * Get derived keyword
         *
         * @param {String} keyword
         * @returns {String}
         */
        getDerived: function (keyword) {
            return helperUtils.sprintf($t('Competitor for keyword %s'), '<span>"' + keyword + '"</span>');
        },

        /**
         * Reset filters
         */
        resetSortingAndFilters: function () {
            self.isFilterChanged = false;
            self.filterByCompetition([...self.defaultFilterByCompetition]);
            self.filterBySearchVolume(self.defaultFilterBySearchVolume);
            self.volumeSort('');
            self.competitionSort('');
            self.intentSort('');
            self.rankSort('');
            self.isFilterChanged = true;
        },

        /**
         * Dismiss
         */
        dismiss: function () {
            keywordProgress.status({});
            keywordProgress.selectedAndCompetitorLoading(false);
            keywordProgress.selectedAndCompetitorProgressVisible(false);
        },

        /**
         * Show more keywords.
         */
        showMore: function () {
            $(".keyword-analysis-competitors-ranking .keyword-row:hidden").slice(0, 10).slideDown();
            if ($(".keyword-analysis-competitors-ranking .keyword-row:hidden").length === 0) {
                this.canShowMore(false);
            }
        },

        /**
         * Check if keyword is selected.
         *
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordSelected: function (keyword) {
            return keywordUtils.isKeywordSelected(keyword);
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },
        
        /**
         * Remove keyword from product.
         * 
         * @param {Object} data
         * @param {Event} event
         * 
         * @returns {void}
         */
        removeKeyword: function (data, event) {
            var element = event.currentTarget,
                parentElement = $(element).closest('tr');

            if (self.isSectionDisabled()) {
                return;
            }

            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.errorMessage('');
            
            let selectedKeyword = this.keyword;
            let keywordToRemove = ko.utils.arrayFirst(
                keywords.targetKeywords(),
                function (data) {
                    return (
                        data.text === selectedKeyword
                    );
                }
            );

            if (keywordToRemove) {
                keywords.targetKeywords.remove(keywordToRemove);
            }

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    remaining_keywords: keywords.targetKeywords()
                        .map(function (keyword) {
                            return { text: keyword.text };
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    $(parentElement).find('.action').removeClass('loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.targetRemoved(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.push(keywordToRemove);
                    }
                }.bind(this)
            });
        },

        /**
         * Add keyword to product.
         * 
         * @param {Object} data
         * @param {Event} event
         * 
         * @returns {void}
         */
        addKeyword: function (data, event) {
            var targetKeywords = self
                    .targetKeywords()
                    .map((item) => item['text']),
                element = event.currentTarget,
                parentElement = $(element).closest('tr'),
                keywordData = {
                    text: this.keyword,
                    values: [],
                    temp: true,
                    search_volume: this.search_volume,
                    competition: this.competition
            };

            if (self.isKeywordsMax() || self.isSectionDisabled()) {
                return;
            }
                
            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            keywords.errorMessage('');
            /* Validate keyword length */
            if (keywordData.text.length > self.rules.maxKeywordLength) {
                keywords.errorMessage('Target keyword is too long.');
                return;
            }
            /* Validate duplicate keyword */
            var isDuplicate = self.targetKeywords().find(keyword => {
                return keyword.text.toLowerCase() === keywordData.text.toLowerCase();
            });
            if (isDuplicate) {
                keywords.errorMessage($t('The entered keyword already exists.'));
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.targetKeywords.push(keywordData);

            $.ajax({
                url: self.keywordAddUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keywords: targetKeywords,
                    keyword: keywordData.text
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    var newKeyword = {};

                    $(parentElement).find('.action').removeClass('loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (response.success) {
                        const oldKeyword = ko.utils.arrayFirst(
                            keywords.targetKeywords(),
                            function (data) {
                                return (
                                    data.text === response.data.text &&
                                    data.temp === true
                                );
                            }
                        );
                        
                        if (oldKeyword) {
                            newKeyword = {...oldKeyword};
                            newKeyword.values = response.data.values;
                            keywords.targetKeywords.replace(oldKeyword, newKeyword);
                        } else {
                            keywords.targetKeywords.push(response.data);
                        }

                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.newTargetExists(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.remove(function (data) {
                            return (
                                data.text === keywordData.text &&
                                data.temp === true
                            );
                        });
                    }
                }
            });

            return true;
        },

        /**
         * Check if keyword is removable.
         * 
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword, false);
        },
    
        /**
         * Refresh data.
         */
        refresh: function () {
            keywords.successMessage('');
            keywordProgress.selectedAndCompetitorLoading(true);
            keywordProgress.selectedAndCompetitorProgressVisible(true);
            keywords.errorMessage('');
            self.resetSortingAndFilters();
            keywordFilters.competitorSortAndFilters({
                competition: ['LOW', 'MEDIUM', 'HIGH'],
                search_volume_minimum: 0,
                search_volume_maximum: null,
                volume_sort: '',
                competition_sort: '',
                intent_sort: '',
                rank_sort: ''
            });
            
            $.ajax({
                url: self.refreshUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    product_name: product.productName(),
                    store_id: product.storeId(),
                    product_url: product.productUrl(),
                    keywords: keywords.manualInput().map(input => input.keyword),
                    target_keywords: keywords.targetKeywords()
                        .map(input => input.text)
                        .filter(keyword => keywordUtils.keywordRemovable(keyword)),
                    location: keywords.location()
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                showAnalysisLoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (response.success) {
                        self.resetSortingAndFilters();
                    } else {
                        keywords.errorMessage(response.message);
                        keywordProgress.selectedAndCompetitorLoading(false);
                        keywordProgress.allSectionsLoading(false);
                        keywordProgress.ideasLoading(false);
                        $('.writetextai-analysis-loading-mask').removeClass('show');
                    }
                }
            });
        },

        /**
         * Save product filter state
         */
        saveFilter: function () {
            var filterBySearchVolumeObj = JSON.parse(self.filterBySearchVolume());

            if (!self.isFilterChanged) {
                return;
            }

            keywords.errorMessage('');
            
            if (self.gettingData !== null) {
                self.gettingData.abort();
            }

            self.gettingData = $.ajax({
                url: self.saveFilterUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    competition: self.filterByCompetition(),
                    search_volume_minimum: filterBySearchVolumeObj.min,
                    search_volume_maximum: filterBySearchVolumeObj.max,
                    competition_sort: self.competitionSort(),
                    volume_sort: self.volumeSort(),
                    rank_sort: self.rankSort(),
                    intent_sort: self.intentSort(),
                    type: 'competitor'
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                showAnalysisLoader: false,
                success: function (response) {
                    if (!response.success) {
                        keywords.errorMessage(response.message);
                    }
                },
                error: function (xhr, status, error) {
                    if (status !== 'abort') {
                        keywords.errorMessage(error);
                    }
                }
            });
        },

        /**
         * Sort keywords by rank.
         */
        sortByRank: function () {
            this.competitionSort('');
            this.volumeSort('');
            this.intentSort('');
            this.rankSort(!this.rankSort());

            this.competitorKeywords(this.competitorKeywords().sort(function (a, b) {
                if (self.rankSort()) {
                    return a.rank_group - b.rank_group;
                } else {
                    return b.rank_group - a.rank_group;
                }
            }));

            this.saveFilter();
        },

        /**
         * Sort keywords by search volume.
         */
        sortBySearchVolume: function () {
            this.competitionSort('');
            this.intentSort('');
            this.volumeSort(!this.volumeSort());
            this.rankSort('');

            this.competitorKeywords(this.competitorKeywords().sort(function (a, b) {
                if (self.volumeSort()) {
                    return a.search_volume - b.search_volume;
                } else {
                    return b.search_volume - a.search_volume;
                }
            }));

            this.saveFilter();
        },

        /**
         * Sort keywords by competition.
         */
        sortByDifficulty: function () {
            this.competitionSort(!this.competitionSort());
            this.volumeSort('');
            this.intentSort('');
            this.rankSort('');

            this.competitorKeywords(this.competitorKeywords().sort(function (a, b) {
                if (a && b && a.competition && b.competition) {
                    var competitionValues = {
                        'LOW': 1,
                        'MEDIUM': 2,
                        'HIGH': 3
                    };

                    var aValue = competitionValues[a.competition.toUpperCase()];
                    var bValue = competitionValues[b.competition.toUpperCase()];

                    if (self.competitionSort()) {
                        return aValue - bValue;
                    } else {
                        return bValue - aValue;
                    }
                } else {
                    return 0;
                }
            }));

            this.saveFilter();
        },

        /**
         * Sort keywords by intent.
         */
        sortByIntent: function () {
            this.competitionSort('');
            this.volumeSort('');
            this.intentSort(!this.intentSort());
            this.rankSort('');

            this.competitorKeywords(this.competitorKeywords().sort(function (a, b) {
                if (self.intentSort()) {
                    return a.intent.localeCompare(b.intent);
                } else {
                    return b.intent.localeCompare(a.intent);
                }
            }));

            this.saveFilter();
        },

        /**
         * Sort keywords by keyword.
         */
        sortByKeyword: function () {
            this.competitionSort('');
            this.volumeSort('');
            this.intentSort('');
            this.rankSort('');

            this.competitorKeywords(this.competitorKeywords().sort(function (a, b) {
                return a.rank_absolute - b.rank_absolute;
            }));

            this.saveFilter();
        }

    });
});
