/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis/ideas',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    product,
    premium,
    prompt,
    totalCredits,
    audience,
    keywordUtils,
    keywordProgress,
    ideasComponent,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generalMessage: $t('A system error has occurred. Please click the "Start AI-powered analysis" button again. If the issue persists, please contact our support team at support@writetext.ai.'),
            newKeyword: '',
            manualKeywords: [],
            manualInputPlaceholder: $t('Add your own keyword here...')
        },

        isManualKeywordsChange: false,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.keyword-analysis-manual-input > *').applyBindings(this);

            keywords.manualInput.subscribe(function (manualKeywords) {
                self.isManualKeywordsChange = true;
                self.manualKeywords(manualKeywords);
                self.isManualKeywordsChange = false;
            });

            self.manualKeywords.subscribe(function (manualKeywords) {
                if (!self.isManualKeywordsChange) {
                    keywords.manualInput(manualKeywords);
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'newKeyword',
                    'manualKeywords'
                ]);

            this.keywordsCount = ko.computed(function () {
                var keywords = this.manualKeywords(),
                    count = keywords ? keywords.length : 0;
                return count + '/' + this.rules.maxSuggestedKeywords;
            }, this);

            this.isKeywordsMax = ko.computed(function () {
                var keywords = this.manualKeywords(),
                    keywordsCount = keywords ? keywords.length : 0;

                return keywordsCount >= this.rules.maxSuggestedKeywords;
            }, this);

            this.targetKeywords = ko.computed(function () {
                return keywords.targetKeywords();
            });

            this.isTargetKeywordsMax = ko.computed(function () {
                var keywords = this.targetKeywords().filter(function (keyword) {
                        return keywordUtils.keywordRemovable(keyword.text);
                }),
                    keywordsCount = keywords ? keywords.length : 0;

                return keywordsCount >= this.rules.maxKeywords;
            }, this);

            this.keywordsChangeOngoing = ko.computed(function () {
                return keywords.keywordsChangeOngoing();
            });

            this.manualKeywordsChangeOngoing = ko.computed(function () {
                return keywords.manualKeywordsChangeOngoing();
            });

            this.isSectionDisabled = ko.computed(function () {
                return keywords.ideasSortFilterOngoing() ||
                    keywords.ideasShowMoreOngoing() ||
                    keywords.keywordsChangeOngoing() ||
                    keywords.manualKeywordsChangeOngoing() ||
                    keywordProgress.ideasLoading() ||
                    keywordProgress.selectedAndCompetitorLoading();
            });

            this.analysisInProgress = keywordUtils.analysisInProgress();

            this.status = ko.computed(function () {
                return keywordProgress.status();
            });

            this.showProgress = ko.computed(function () {
                return Object.keys(this.status()).length > 0 &&
                    keywordProgress.ideasProgressVisible() &&
                    this.status().status !== 'Completed';
            }, this);

            this.getMessage = keywordUtils.getMessage(this);

            this.getCount = keywordUtils.getCount(this);

            this.getPercent = keywordUtils.getPercent(this);

            this.showRefresh = ko.computed(function () {
                return keywords.showIdeasRefresh();
            }, this);

            this.initialized = ko.computed(function () {
                return keywords.initialized();
            });

            this.isRefreshing = ko.computed(function () {
                return keywordProgress.ideasLoading();
            });

            this.stale = ko.computed(function () {
                return keywords.stale();
            });

            this.newKeywordExists = ko.computed(function () {
                return keywords.newKeywordExists();
            });

            this.needRefresh = ko.computed(function () {
                return keywords.needRefresh();
            });

            return this;
        },

        /**
         * Refresh data.
         */
        refresh: function () {
            ideasComponent.prototype.refresh();
        },

        /**
         * Dismiss
         */
        dismiss: function () {
            keywordProgress.status({});
            keywordProgress.ideasLoading(false);
            keywordProgress.ideasProgressVisible(false);
        },

        /**
         * Check if keyword is selected.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        isKeywordSelected: function (keyword) {
            return keywordUtils.isKeywordSelected(keyword);
        },

        /**
         * Get Keyword input note.
         */
        getKeywordInputNote: function () {
            return $t('Enter up to %s keywords. Press <ENTER> or use a comma to separate each keyword.').replace('%s', self.rules.maxSuggestedKeywords);
        },

        /**
         * Get keyword length.
         */
        getKeywordLength: function () {
            return (
                this.newKeyword().length +
                '/' +
                this.rules.maxKeywordLength +
                ' ' +
                $t('Char')
            );
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },

        /**
         * Remove keyword from product.
         *
         * @param {object} event
         * @param {object} data
         * 
         * @returns {void}
         */
        removeKeyword: function (event, data) {
            var keyword = data,
                element = event.currentTarget,
                parentElement = $(element).closest('tr');

            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.errorMessage('');

            let targetKeywordToRemove = keywords.targetKeywords.filter(function (targetKeyword) {
                return targetKeyword.text === keyword.keyword;
            });

            if (targetKeywordToRemove.length > 0) {
                keywords.targetKeywords.remove(function (targetKeyword) {
                    return targetKeyword.text === keyword.keyword;
                });
            }

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    remaining_keywords: keywords.targetKeywords()
                        .filter(function (keyword) {
                            return keyword.text !== self.keyword;
                        })
                        .map(function (keyword) {
                            return { text: keyword.text };
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    $(parentElement).find('.action').removeClass('loading');
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.targetRemoved(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.push(targetKeywordToRemove);
                    }
                }.bind(this)
            });
        },

        /**
         * Delete manual keyword.
         *
         * @param {object} data
         * @returns {void}
         */
        deleteManualKeyword: function (data) {
            if (self.isKeywordSelected(data.keyword)) {
                return;
            }
            
            keywords.errorMessage('');
            self.manualKeywords.remove(function (manualKeyword) {
                return (
                    manualKeyword.keyword === data.keyword
                );
            });
            $.ajax({
                url: self.ManualKeywordDeleteUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keyword: data.keyword,
                    location: keywords.location()
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showManualKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (!response.success) {
                        keywords.errorMessage(response.message);
                        keywords.manualInput.push(data)
                    } else {
                        keywords.targetRemoved(true);
                    }
                }
            });
        },

        /**
         * Check if keyword is removable.
         *
         * @param {string} keyword
         * @returns {boolean}
         */
        keywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword);
        },

        /**
         * Auto lower case.
         */
        autoLowerCase: function () {
            self.newKeyword(self.newKeyword().toLowerCase());
        },

        /**
         * Add manual keyword.
         *
         * @param {object} binding
         * @param {object} event
         *
         * @returns {Boolean|undefined}
         */
        addManualKeyword: function (binding, event) {
            var sanitizedKeyword = self.newKeyword().replace(/<\/?[^>]+(>|$)/g, ""),
                keywordData = {
                    keyword: sanitizedKeyword.toLowerCase().trim()
                };

            if (
                event.keyCode === 13 ||
                event.keyCode === 44
            ) {
                if (!premium.isPremium()) {
                    self.closeModal();
                    return;
                }

                /* Don't add empty keywords */
                if (keywordData.keyword.length < 1) {
                    return;
                }

                keywords.errorMessage('');

                /* Validate keyword length */
                if (keywordData.keyword.length > self.rules.maxKeywordLength) {
                    keywords.errorMessage('Target keyword is too long.');
                    return;
                }

                /* Validate duplicate keyword */
                var isDuplicate = self.manualKeywords().find(keyword => {
                    return keyword.keyword.toLowerCase() === keywordData.keyword.toLowerCase();
                });
                if (isDuplicate) {
                    keywords.errorMessage($t('The entered keyword already exists.'));
                    return;
                }

                /* Validate product name */
                if (!keywordUtils.keywordRemovable(keywordData.keyword, false)) {
                    keywords.errorMessage($t('The product name is considered by default in generating text. Please add a different keyword.'));
                    return;
                }

                self.manualKeywords.push(keywordData);

                /* Clear new keyword input */
                if (event.type !== 'click') {
                    self.newKeyword('');
                }

                $.ajax({
                    url: self.ManualKeywordAddUrl,
                    type: 'POST',
                    data: {
                        product_id: product.productId(),
                        store_id: product.storeId(),
                        keyword: keywordData.keyword,
                        location: keywords.location()
                    },
                    dataType: 'json',
                    showWriteTextAILoader: true,
                    showManualKeywordsChangeLoader: true,
                    hideWriteTextTitleLoader: true,
                    success: function (response) {
                        if (typeof response.is_premium !== 'undefined') {
                            premium.isPremium(response.is_premium);
                            prompt.isPremium(response.is_premium);
                            totalCredits.totalCredits(response.total_credits);
                        }

                        if (response.success) {
                            self.newKeyword('');
                            keywords.newKeywordExists(true);
                        } else {
                            keywords.errorMessage(response.message);
                            self.manualKeywords.remove(function (data) {
                                return (
                                    data.keyword === keywordData.keyword
                                );
                            });
                        }
                    }
                });

                if (event.keyCode === 44) {
                    return;
                }
            }

            return true;
        },

        /**
         * Add keyword.
         * 
         * @param {object} data
         * @param {Event} event
         *
         * @returns {void}
         */
        addKeyword: function (data, event) {
            var targetKeywords = keywords
                    .targetKeywords()
                    .map((item) => item['text']),
                element = event.currentTarget,
                parentElement = $(element).closest('tr'),
                keywordData = {
                    text: this.keyword,
                    values: [],
                    temp: true,
                    search_volume: this.search_volume,
                    competition: this.competition,
                    rank_group: this.rank_group,
                    intent: this.intent
            };

            if (self.isTargetKeywordsMax()) {
                return;
            }
                
            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            keywords.errorMessage('');
            /* Validate keyword length */
            if (keywordData.text.length > self.rules.maxKeywordLength) {
                keywords.errorMessage('Target keyword is too long.');
                return;
            }
            
            /* Validate duplicate keyword */
            var isDuplicate = keywords.targetKeywords().find(keyword => {
                return keyword.text.toLowerCase() === keywordData.text.toLowerCase();
            });
            if (isDuplicate) {
                keywords.errorMessage($t('The entered keyword already exists.'));
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.targetKeywords.push(keywordData);

            $.ajax({
                url: self.keywordAddUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keywords: targetKeywords,
                    keyword: keywordData.text
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                showKeywordsChangeLoader: true,
                hideWriteTextTitleLoader: true,
                success: function (response) {
                    var newKeyword = {};

                    $(parentElement).find('.action').removeClass('loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (response.success) {
                        const oldKeyword = ko.utils.arrayFirst(
                            keywords.targetKeywords(),
                            function (data) {
                                return (
                                    data.text === response.data.text &&
                                    data.temp === true
                                );
                            }
                        );
                        
                        if (oldKeyword) {
                            newKeyword = {...oldKeyword};
                            newKeyword.values = response.data.values;
                            keywords.targetKeywords.replace(oldKeyword, newKeyword);
                        } else {
                            keywords.targetKeywords.push(response.data);
                        }

                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.newTargetExists(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.remove(function (data) {
                            return (
                                data.text === keywordData.text &&
                                data.temp === true
                            );
                        });
                    }
                }
            });

            return true;
        },
    });
});
