/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'underscore',
    'Magento_Ui/js/grid/massactions',
    'WriteTextAI_WriteTextAI/js/modal/confirm',
    'WriteTextAI_WriteTextAI/js/model/gallery',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/grid/notifications',
    'WriteTextAI_WriteTextAI/js/model/grid/error-messages',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/grid/transfer',
    'WriteTextAI_WriteTextAI/js/model/grid/upload-images',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'mage/translate',
    'WriteTextAI_WriteTextAI/js/grid/reload',
    'WriteTextAI_WriteTextAI/js/utils/grid/actions',
    'WriteTextAI_WriteTextAI/js/model/grid/reference-products',
    'WriteTextAI_WriteTextAI/js/model/grid/invalid-image-popup',
    'mage/validation'
], function (
    $,
    ko,
    _,
    Massactions,
    confirm,
    galleryModel,
    totalCredits,
    premium,
    notifications,
    errorMessagesModel,
    prompt,
    transfer,
    uploadImages,
    totalCreditsModel,
    $t,
    reloadGrid,
    gridUtils,
    referenceProducts,
    gridInvalidImagePopupModel
) {
    'use strict';

    return Massactions.extend({
        defaults: {
            template: 'WriteTextAI_WriteTextAI/grid/actions',
            generateFormSelector: '#bulk-generate-form',
            transferFormSelector: '#bulk-transfer-form',
            statusRunning: 'Running',
            defaultStoreId: 0,
            currentUser: {},
            isPremium: true,
            totalCredits: 0,
            textTypes: [
                'page_title',
                'page_description',
                'product_description',
                'short_product_description',
                'open_graph'
            ]
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            this.isPremium.subscribe(function (isPremium) {
                if (typeof isPremium !== 'undefined') {
                    prompt.isPremium(isPremium);
                }
            });

            this.currentUser.subscribe(function (currentUser) {
                if (currentUser && Object.keys(currentUser).length !== 0) {
                    gridInvalidImagePopupModel.currentUser(currentUser);
                    this.continueBulkTransfer();
                }
            }, this);

            this.totalCredits.subscribe(function (totalCredits) {
                totalCreditsModel.totalCredits(totalCredits);
            }, this);
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'currentUser',
                    'isPremium',
                    'productImages',
                    'totalCredits',
                    'ids'
                ]);

            this.isDisabled = ko.computed(function () {
                if (!this.isPremium()) {
                    return true;
                }
                return notifications.statuses().some((status) => {
                    return (
                        status.status === this.statusRunning &&
                        status.user === this.currentUser().email
                    );
                }, this);
            }, this);

            return this;
        },

        /**
         * Continue bulk transfer
         */
        continueBulkTransfer: function () {
            try {
                notifications.statuses().forEach(status => {
                    if (status.status === this.statusRunning &&
                        status.user === this.currentUser().email &&
                        status.transfer === true &&
                        status.queuedIds.length > 0
                    ) {
                        let action = {},
                            data = {
                                excludeMode: false,
                                excluded: [],
                                params: {
                                    fields: status.continueParams.fields,
                                    stores: status.continueParams.stores,
                                    namespace: "products_grid_listing",
                                    filters: {
                                        store_id: status.continueParams.store_filter
                                    }
                                },
                                /** always set completed first so the index is correct */
                                selected: [...status.completedIds, ...status.queuedIds],
                                total: status.queuedIds.length + status.completedIds.length
                        },
                            settings = {
                                stores: status.continueParams.stores,
                                fields: status.continueParams.fields
                        },
                            index = status.completedIds.length;
                            
                        transfer.completedIds(status.completedIds);
                        transfer.queueIds(status.queuedIds);
                        this.bulkTransfer(action, data, settings, index);
                    }
                }, this);
            } catch (e) {
                return;
            }
        },

        /** @inheritdoc */
        applyAction: function (actionIndex) {
            if (this.isDisabled()) {
                return;
            }

            return this._super(actionIndex);
        },

        /**
         * Default action callback. Sends selections data
         * via POST request.
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         *
         * @returns {void}
         */
        defaultCallback: function (action, _data) {
            var settings = {};
            var data = JSON.parse(JSON.stringify(_data));

            errorMessagesModel.messages([]);
            switch (action.type) {
                case 'generate':
                    try {
                        settings = gridUtils.convertForm($(this.generateFormSelector));
                        this.bulkGenerate(action, data, settings);
                    } catch (e) {
                        errorMessagesModel.messages.push(window.wtaGeneralErrorMessage);
                        console.log('bulk generate failed', e);
                    }
                    break;
                case 'transfer':
                    try {
                        settings = gridUtils.convertForm($(this.transferFormSelector));
                        this.transferable(action, data, settings);
                    } catch (e) {
                        errorMessagesModel.messages.push(window.wtaGeneralErrorMessage);
                        console.log('bulk transfer failed', e);
                    }
                    break;
                default:
                    this._super(action, data);
            }
        },

        /**
         * Bulk generate
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        bulkGenerate: function (action, data, settings) {
            if ((this.textTypes.some(textType => settings.fields.includes(textType)) &&
                    settings.attributes &&
                    settings.attributes.includes('thumbnail')
                ) || settings.fields.includes('alt_text')
            ) {
                this.uploadImages(action, data, settings);
            } else {
                this.bulkGenerateRequest(action, data, settings);
            }
        },

        /**
         * Upload images request
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        uploadImages: function (action, data, settings) {
            $('.writetextai-upload-image-loader').modal('openModal');
            uploadImages.failedThumbnails([]);
            uploadImages.failedProducts([]);
            uploadImages.successThumbnails([]);
            this.uploadThumbnails(action, data, settings);
        },

        /**
         * Upload thumbnails
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         * @param {Number} index - Index of the current chunk.
         *
         * @returns {void}
         */
        uploadThumbnails: function (action, data, settings, index = 0) {
            var selections = {};
            var endIndex = Math.min(index + 10, data.selected.length);

            if (this.textTypes.some(textType => settings.fields.includes(textType)) &&
                settings.attributes &&
                settings.attributes.includes('thumbnail') &&
                index < data.selected.length
            ) {
                $.ajax({
                    url: this.uploadThumbnailsUrl,
                    type: 'POST',
                    data: selections,
                    data: {
                        selected: data.selected.slice(index, endIndex),
                        store_id: referenceProducts.storeId()
                    },
                    dataType: 'json',
                    showLoader: false,
                    success: function (response) {
                        if (!response.success) {
                            $('.writetextai-upload-image-loader').modal('closeModal');
                            $('.modals-overlay.writetextai-upload-image-loader-overlay').remove();
                            errorMessagesModel.messages.push(response.message);
                            if (response.unauthorized) {
                                window.location.href = response.login_url;
                            }
                        } else {
                            uploadImages.failedThumbnails.push(...response.failed_thumbnails);
                            uploadImages.failedProducts.push(...response.failed_products);
                            uploadImages.successThumbnails.push(...response.success_thumbnails);

                            /** Upload next chunk */
                            this.uploadThumbnails(action, data, settings, endIndex);
                        }
                    }.bind(this)
                });
            } else {
                this.uploadAltTextImages(action, data, settings);
            }
        },

        /**
         * Upload alt text images
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        uploadAltTextImages: function (action, data, settings) {
            var ids = this.ids(); /** Get ids of products in grid with/without filter */
            var productImages = this.productImages();
            var selectedProductImages = Object.keys(productImages)
                .filter(productId => {
                    if (data.excludeMode) {
                        return ids.includes(productId) && !data.excluded.includes(productId);
                    } else {
                        return ids.includes(productId) && data.selected.includes(productId);
                    }
                })
                .flatMap(productId => productImages[productId].split(',').map(image => ({
                    image: image,
                    entityId: productId
                })));

            /** Split images into chunks of 10 */
            var imageChunks = [];
            for (var i = 0; i < selectedProductImages.length; i += 10) {
                imageChunks.push(selectedProductImages.slice(i, i + 10));
            }

            galleryModel.productsUploading(data.selected);
            uploadImages.failedImages([]);
            uploadImages.failedImageIds([]);
            uploadImages.successImages([]);

            /** Start uploading the first chunk */
            this.uploadRequest(action, data, settings, imageChunks, 0);
        },
        
        /**
         * Upload images request
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         * @param {Array} imageChunks - Array of image chunks.
         * @param {Number} index - Index of the current chunk.
         *
         * @returns {void}
         */
        uploadRequest: function (action, data, settings, imageChunks, index) {
            if (settings.fields.includes('alt_text') &&
                index < imageChunks.length
            ) {
                $.ajax({
                    url: this.uploadImagesUrl,
                    type: 'POST',
                    data: {
                        images: imageChunks[index],
                        store_id: referenceProducts.storeId()
                    },
                    dataType: 'json',
                    showLoader: false
                }).done(function (response) {
                    if (!response.success) {
                        $('.writetextai-upload-image-loader').modal('closeModal');
                        $('.modals-overlay.writetextai-upload-image-loader-overlay').remove();
                        errorMessagesModel.messages.push(response.message);
                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                    } else {
                        uploadImages.failedImages.push(...response.failed_images);
                        uploadImages.failedImageIds.push(...response.failed_image_ids);
                        uploadImages.successImages.push(...response.success_images);

                        /** Upload next chunk */
                        this.uploadRequest(action, data, settings, imageChunks, index + 1);
                    }
                }.bind(this));
            } else {
                galleryModel.productsUploading([]);
                $('.writetextai-upload-image-loader').modal('closeModal');
                $('.modals-overlay.writetextai-upload-image-loader-overlay').remove();

                if (this.validateInvalidImages()) {
                    this.bulkGenerateRequest(action, data, settings);
                } else {
                    gridInvalidImagePopupModel.data(data);
                    gridInvalidImagePopupModel.settings(settings);
                    $('.writetextai-grid-invalid-image-popup').modal('openModal');
                }
            }
        },

        /**
         * Validate invalid images
         */
        validateInvalidImages: function () {
            var failedImages = uploadImages.failedImages(),
                failedThumbnails = uploadImages.failedThumbnails();

            return failedThumbnails.length === 0 && failedImages.length === 0;
        },

        /**
         * Bulk generate request
         * via POST request.
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        bulkGenerateRequest: function (action, data, settings) {
            var itemsType = '',
                selections = {};

            notifications.addNewGenerateStatus(
                data.selected,
                this.currentUser(),
                data
            );

            _.extend(data.params, settings || {});

            itemsType = data.excludeMode ? 'excluded' : 'selected';

            selections[itemsType] = data[itemsType];

            if (!selections[itemsType].length) {
                selections[itemsType] = false;
            }

            _.extend(selections, data.params || {});

            /* set default store filter */
            if (!selections['filters']['store_id']) {
                selections['filters']['store_id'] = this.defaultStoreId;
            }

            $.ajax({
                url: action.url,
                type: 'POST',
                data: selections,
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        notifications.statuses.remove(function (status) {
                            return status.id === notifications.tempId;
                        });
                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                        if (response.reload) {
                            this.gridReload();
                        }
                    } else {
                        this.gridReload();
                    }
                }.bind(this)
            });
        },

        /**
         * Bulk transfer request
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        transferable: function (action, data, settings) {
            var itemsType = '',
                selections = {};

            _.extend(data.params, settings || {});

            itemsType = data.excludeMode ? 'excluded' : 'selected';

            /** Fixes for select all not actually selecting all */
            if (data && data.excludeMode) {
                data.selected = this.ids().filter(id => !data.excluded.includes(id));
                selections.selected = this.ids().filter(id => !data.excluded.includes(id));
            }

            selections[itemsType] = data[itemsType];

            if (!selections[itemsType].length) {
                selections[itemsType] = false;
            }

            _.extend(selections, data.params || {});

            /* set default store filter */
            if (!selections['filters']['store_id']) {
                selections['filters']['store_id'] = this.defaultStoreId;
            }

            $.ajax({
                url: this.transferableUrl,
                type: 'POST',
                data: selections,
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        errorMessagesModel.messages.push(response.message);
                        if (response.unauthorized) {
                            window.location.href = response.login_url;
                        }
                        if (response.reload) {
                            this.gridReload();
                        }
                    } else {
                        transfer.queueIds(data.selected);
                        transfer.completedIds([]);
                        
                        notifications.addNewTransferStatus(
                            data.selected,
                            this.currentUser(),
                            data
                        );

                        transfer.cancelBulkTransfer(false);
                        this.saveBulkTransferProgress(data, settings);
                        this.bulkTransfer(action, data, settings, 0);
                    }
                }.bind(this)
            });
        },

        /**
         * Bulk transfer request
         *
         * @param {Object} action - Action data.
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         * @param {Number} index - Index of the current chunk.
         *
         * @returns {void}
         */
        bulkTransfer: function (action, data, settings, index) {
            var itemsType = '',
                selections = {};

            if (index < data.selected.length) {
                _.extend(data.params, settings || {});

                itemsType = data.excludeMode ? 'excluded' : 'selected';

                selections[itemsType] = data[itemsType];

                if (!selections[itemsType].length) {
                    selections[itemsType] = false;
                }

                _.extend(selections, data.params || {});

                /* set default store filter */
                if (!selections['filters']['store_id']) {
                    selections['filters']['store_id'] = this.defaultStoreId;
                }
                
                let productIds = [...data.selected];
                let completedIds = [...transfer.completedIds()];
                let queueIds = transfer.queueIds().filter(item => item !== productIds[index]);

                completedIds.push(productIds[index]);

                transfer.completedIds(completedIds);
                transfer.queueIds(queueIds);

                selections.selected = [data.selected[index]];

                $.ajax({
                    url: this.bulkTransferUrl,
                    type: 'POST',
                    data: selections,
                    dataType: 'json',
                    showLoader: false,
                    success: function (response) {
                        if (typeof response.is_premium !== 'undefined') {
                            premium.isPremium(response.is_premium);
                            prompt.isPremium(response.is_premium);
                            totalCredits.totalCredits(response.total_credits);
                        }
                        if (!response.success) {
                            if (response.unauthorized) {
                                window.location.href = response.login_url;
                            }
                            if (response.reload) {
                                this.gridReload();
                            }
                        }
                        if (transfer.cancelBulkTransfer() === false) {
                            notifications.updateTransferStatus(
                                productIds,
                                this.currentUser().email,
                                queueIds,
                                completedIds
                            );
                            this.saveBulkTransferProgress(data, settings);
                            this.bulkTransfer(action, data, settings, index + 1);
                        } else {
                            notifications.updateTransferStatus(
                                productIds,
                                this.currentUser().email,
                                [],
                                completedIds
                            );
                        }
                    }.bind(this)
                });
            } else {
                this.gridReload();
            }
        },

        /**
         * Save bulk transfer progress
         *
         * @param {Object} data - Selections data.
         * @param {Object} settings - Form settings.
         *
         * @returns {void}
         */
        saveBulkTransferProgress: function (data, settings) {
            $.ajax({
                url: this.saveTransferProgressUrl,
                type: 'POST',
                data: {
                    queue_ids: transfer.queueIds(),
                    completed_ids: transfer.completedIds(),
                    store_id: data.params.filters['store_id'] ?? this.defaultStoreId,
                    stores: settings.stores,
                    fields: settings.fields,
                    user: this.currentUser().email
                },
                dataType: 'json',
                showLoader: false,
                success: function (response) {
                    if (!response.success) {
                        console.log(response.message);
                    }
                }
            });
        },

        /**
         * Reload grid.
         */
        gridReload: function () {
            var showLoader = false;

            reloadGrid.reloadUIComponent(
                'products_grid_listing.products_grid_listing_data_source',
                showLoader
            );
        },

        /**
         * Update selection images count.
         *
         * @param {Object} data - Selections data.
         * @returns {void}
         */
        updateSelectionImagesCount: function (data) {
            var ids = this.ids();
            var productImages = this.productImages();
            var selectedProductImages = Object.keys(productImages)
                .filter(productId => {
                    if (data.excludeMode) {
                        return ids.includes(productId) && !data.excluded.includes(productId);
                    } else {
                        return ids.includes(productId) && data.selected.includes(productId);
                    }
                })
                .map(productId => productImages[productId].split(','));

            selectedProductImages = [].concat(...selectedProductImages);

            prompt.selectionImagesCount(selectedProductImages.length);
        },

        /**
         * Shows actions' confirmation window.
         *
         * @param {Object} action - Actions' data.
         * @param {Function} callback - Callback that will be
         *      invoked if action is confirmed.
         *
         * @returns {void}
         */
        _confirm: function (action, callback) {
            var confirmData = action.confirm,
                confirmMessage = confirmData.message,
                buttonText = '',
                modalClass = '',
                formSelector = '',
                data = this.getSelections(),
                total = data.total ? data.total : 0;

            prompt.selectionsCount(total);

            this.updateSelectionImagesCount(data);

            if (action.type !== 'generate' && action.type !== 'transfer') {
                this._super(action, callback);
                return;
            }

            if (action.type === 'generate') {
                buttonText = confirmData.button;
                modalClass = 'writetextai-modal bulk-generate-popup confirm';
                formSelector = this.generateFormSelector;
            }

            if (action.type === 'transfer') {
                buttonText = $t('Transfer');
                modalClass = 'writetextai-modal bulk-transfer-popup transfer';
                formSelector = this.transferFormSelector;
            }

            confirm({
                title: confirmData.title,
                content: confirmMessage,
                modalClass: modalClass,
                actions: {
                    confirm: callback,
                    cancel: function () {
                        $('body').removeClass('_has-modal');
                    }
                },
                buttons: [{
                    text: $t('Cancel'),
                    class: 'action-secondary action-dismiss',
                    click: function (event) {
                        this.closeModal(event);
                    }
                }, {
                    text: buttonText,
                    class: 'action-primary action-accept',
                    click: function (event) {
                        if (
                            $(formSelector).data('validator') &&
                            $(formSelector).validation() &&
                            $(formSelector).validation('isValid')
                        ) {
                            this.closeModal(event, true);
                            $('body').removeClass('_has-modal');
                        }
                    }
                }]
            });
        }
    });
});
