/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress'
], function ($, ko, keywords, product, keywordProgress) {
    'use strict';

    /**
     * Check if keyword is removable.
     *
     * @param {String} keyword
     * @param {Boolean} caseSensitive
     * 
     * @returns {Boolean}
     */
    function keywordRemovable(keyword, caseSensitive = true)
    {
        var productName = product.productName(),
            parsedKeyword =  $($.parseHTML(keyword)).text();

        if (!caseSensitive) {
            productName = productName.toLowerCase();
            parsedKeyword = parsedKeyword.toLowerCase();
        }

        return parsedKeyword !== productName;
    }

    /**
     * Check if keyword is selected.
     *
     * @param {String} keyword
     * @returns {Boolean}
     */
    function isKeywordSelected(keyword)
    {
        return keywords.targetKeywords()
            .filter(targetKeyword => keywordRemovable(targetKeyword.text))
            .some(function (targetKeyword) {
                return targetKeyword.text === keyword;
            });
    }

    /**
     * Get message.
     *
     * @param {Object} self
     * @returns {String}
     */
    function getMessage(self)
    {
        return ko.computed(function () {
            const status = self.status();
            const statusDisplay = status.status_display;
            const statusState = status.status;
            const taskStatusMessage = status.task_status_message;

            if (!Object.keys(status).length) {
                return '';
            }

            if (statusState === 'Failed') {
                return statusDisplay || self.generalMessage;
            }

            return statusDisplay || 'Completed';
        });
    }

    /**
     * Get count.
     *
     * @param {Object} self
     * @returns {String}
     */
    function getCount(self)
    {
        return ko.computed(function () {
            if (Object.keys(self.status()).length === 0) {
                return '';
            }

            return self.status().completed + '/' + self.status().total;
        });
    }

    /**
     * Get percent.
     *
     * @param {Object} self
     * @returns {String}
     */
    function getPercent(self)
    {
        return ko.computed(function () {
            if (Object.keys(self.status()).length === 0) {
                return 0;
            }
            
            return (self.status().completed / self.status().total) * 100;
        });
    }
    
    /**
     * Initialize toggle SERP popup.
     *
     * @param {String} selector
     * @param {String} popupSelector
     *
     * @returns {void}
     */
    function initToggleSerpPopup(selector, popupSelector)
    {
        $('.writetextai-keyword-analysis-content').on('click', selector + ' .has-popup', function (event) {
            if ($(event.target).closest(popupSelector).length) {
                return;
            }
            $(selector + ' .has-popup')
                .not(this)
                .removeClass('show-popup');
            $(this).toggleClass('show-popup');
        });
        $('.writetextai-keyword-analysis-content').click(function (event) {
            if (!$(event.target).closest(selector + ' .has-popup').length) {
                $(selector + ' .has-popup').removeClass('show-popup');
            }
        });
    }

    /**
     * Check if analysis is in progress.
     */
    function analysisInProgress()
    {
        return ko.computed(function () {
            return keywordProgress.selectedAndCompetitorLoading() ||
                keywordProgress.allSectionsLoading() ||
                keywordProgress.ideasLoading() ||
                keywords.fetchingData() ||
                keywords.keywordsChangeOngoing() ||
                keywords.ideasShowMoreOngoing() ||
                keywords.ideasSortFilterOngoing() ||
                keywords.manualKeywordsChangeOngoing();
        });
    }

    return {
        isKeywordSelected: isKeywordSelected,
        keywordRemovable: keywordRemovable,
        getMessage: getMessage,
        getCount: getCount,
        getPercent: getPercent,
        initToggleSerpPopup: initToggleSerpPopup,
        analysisInProgress: analysisInProgress
    };
});
